import matplotlib.pyplot as plt
import numpy as np
from whacc import utils
from whacc.utils import info
#
# color=["#E69F00", "#56B4E9", "#009E73", "#0072B2", "#D55E00", "#CC79A7", "#F0E442"]
# for i, k in enumerate(color):
#     plt.plot([0, 1], [i, i], color = k)
#
#
# CB91_Blue = '#2CBDFE'
# CB91_Green = '#47DBCD'
# CB91_Pink = '#F3A0F2'
# CB91_Purple = '#9D2EC5'
# CB91_Violet = '#661D98'
# CB91_Amber = '#F5B14C'
#
# color_list = [CB91_Blue, CB91_Pink, CB91_Green, CB91_Amber,
#               CB91_Purple, CB91_Violet]
# plt.rcParams['axes.prop_cycle'] = plt.cycler(color=color_list)
#
# plt.figure()
# for i, k in enumerate(color_list):
#     plt.plot([0, 1], [i, i], k)
#
#
#
# CB91_Blue = '#2CBDFE'
# CB91_Green = '#47DBCD'
# CB91_Pink = '#F3A0F2'
# CB91_Purple = '#9D2EC5'
# CB91_Violet = '#661D98'
# CB91_Amber = '#F5B14C'
#
# color_list = [CB91_Blue, CB91_Pink, CB91_Green, CB91_Amber,
#               CB91_Purple, CB91_Violet]
# plt.rcParams['axes.prop_cycle'] = plt.cycler(color=color_list)
#

import seaborn as sns

sns.set(rc={
 'axes.axisbelow': False,
 'axes.edgecolor': 'lightgrey',
 'axes.facecolor': 'None',
 'axes.grid': False,
 'axes.labelcolor': 'dimgrey',
 'axes.spines.right': False,
 'axes.spines.top': False,
 'figure.facecolor': 'white',
 'lines.solid_capstyle': 'round',
 'patch.edgecolor': 'w',
 'patch.force_edgecolor': True,
 'text.color': 'dimgrey',
 'xtick.bottom': False,
 'xtick.color': 'dimgrey',
 'xtick.direction': 'out',
 'xtick.top': False,
 'ytick.color': 'dimgrey',
 'ytick.direction': 'out',
 'ytick.left': False,
 'ytick.right': False})
sns.set_context("notebook", rc={"font.size":16,
                                "axes.titlesize":20,
                                "axes.labelsize":18})

fig, ax = plt.subplots(1, figsize=(4, 4))
for i, i2 in enumerate([3,0,1,2]):
  # line = ax.bar(i, np.random.rand(1))
  line = ax.plot(i, np.random.rand(1))
  print(line.get_color())


for i, i2 in enumerate([3,0,1,2]):
    plt.plot([0, 1], [i, i])


col = {}

col['base_models'] = []



# from google.colab import drive
# drive.mount('/content/gdrive')
# drive._mount('/content/gdrive')
from whacc import model_maker

from whacc.model_maker import *


import h5py

from tensorflow.keras.layers import Dense, LSTM, Flatten, TimeDistributed, Conv2D, Dropout
from tensorflow.keras.models import Model
from tensorflow.keras import Sequential
from tensorflow import keras
from tensorflow.keras import applications


from tqdm.notebook import tqdm
from tqdm import tqdm
import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
from datetime import datetime
import pytz
import json
from math import isclose, sqrt
from IPython import display

"""
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
"""

def send_text(mess):#keep
    print('oops texting failed but the code goes on')


def save_obj(obj, name):#keep
    with open(name + '.pkl', 'wb') as f:
        pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)


def load_obj(name):#keep
    with open(name + '.pkl', 'rb') as f:
        return pickle.load(f)

def foo_rename(instr):#keep
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]

# bd2 = "/content/gdrive/My Drive/colab_data2/"
# all_models_directory = bd2+"/model_testing/all_data/all_models/"
# all_data = load_model_data(all_models_directory)


# save_obj(all_data, '/content/all_data2')
all_data = load_obj('/Users/phil/Downloads/all_data')
"""
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
"""
train_and_val_dir = '/Users/phil/Desktop/content/ALL_RETRAIN_H5_data____temp'
srs_TandV_h5s, dst_TandV_h5s = utils.copy_file_filter(
    foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/small_h5s/data'),
    train_and_val_dir,
    keep_strings='/3lag/',
    remove_string='.zip',
    overwrite=True,
    just_print_what_will_be_copied=False, return_list_of_files=True)



"""# download the retrain augmented and OG files to local """
train_and_val_dir = '/Users/phil/Desktop/content/ALL_RETRAIN_H5_data____temp'
srs_TandV_h5s, dst_TandV_h5s = utils.copy_file_filter(
    foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/small_h5s/data'),
    train_and_val_dir,
    keep_strings='/3lag/',
    remove_string='.zip',
    overwrite=True,
    just_print_what_will_be_copied=False, return_list_of_files=True)

h5_imgs, h5_labels, inds_of_files = utils.copy_alt_labels_based_on_directory(srs_TandV_h5s)
for i in inds_of_files:
    utils.copy_over_all_non_image_keys(h5_labels[i], dst_TandV_h5s[i])

base_test_dir = '/Users/phil/Desktop/content/DATA_FULL____temp'
srs_test_h5s, dst_test_h5s = utils.copy_file_filter(
    foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data/test_data/small_h5s'),
    base_test_dir,
    keep_strings='/3lag/',
    remove_string='.zip',
    overwrite=True,
    just_print_what_will_be_copied=False, return_list_of_files=True)

h5_imgs, h5_labels, inds_of_files = utils.copy_alt_labels_based_on_directory(srs_test_h5s)
for i in inds_of_files:
    utils.copy_over_all_non_image_keys(h5_labels[i], dst_test_h5s[i])

"""#trasnform h5s to LSTM H5s"""

base_dir = '/Users/phil/Desktop/content/'
for h5 in utils.lister_it(utils.get_h5s(base_dir), keep_strings='____temp'):  # only go through the non converted h5s
    # utils.print_h5_keys(h5)
    h5_out = ''.join(h5.split('____temp'))
    print(h5, h5_out)
    image_tools.convert_h5_to_LSTM_h5(h5, h5_out)
    utils.copy_over_all_non_image_keys(h5, h5_out)

train_and_val_dir = ''.join(train_and_val_dir.split('____temp')) + os.sep
base_test_dir = ''.join(base_test_dir.split('____temp')) + os.sep

"""
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
"""
def inds_sorted_data(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through
  """
    log_ind = np.where(key_name == a['logs_names'])[0][0]
    val_list = a['all_logs'][:, log_ind]
    if max_or_min == 'max':
        max_arg_sort = np.flip(np.argsort(val_list)) + 1
    elif max_or_min == 'min':
        max_arg_sort = np.argsort(val_list) + 1
    else:
        raise ValueError("""max_or_min must be a string set to 'max' or 'min'""")
    return max_arg_sort


def sorted_loadable_epochs(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through
  """
    arg_sort_inds = inds_sorted_data(data, key_name, max_or_min)
    arg_sort_inds[np.argmax(arg_sort_inds)] = -1
    saved_epoch_numbers = np.asarray(list(data['info']['epoch_dict'].keys()))
    sorted_loadable_epochs_out = []
    for k in arg_sort_inds:
        if k in saved_epoch_numbers:
            sorted_loadable_epochs_out.append(k)
    return sorted_loadable_epochs_out


def get_automated_model_info_TL(BASE_H5, image_source_h5_directory_ending, test_data_dir, data_string_key="data"):#keep
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + data_string_key + a
    base_dir_all_h5s = BASE_H5 + a + data_string_key + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    # pdb.set_trace()
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    # labels_dir = base_data_dir + a + "ALT_LABELS" + a
    # print('\nFOR LABELS,0 is train set, 1 is val set')
    # label_h5_list = utils.get_h5s(labels_dir)
    # print('\nSelect from the following label structures...')
    # print(labels_dir)
    # label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    # h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    # h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()



def re_build_model_TL(model_name_str, class_numbers, base_learning_rate=0.00001,
                      dropout_val=None, IMG_SIZE=96, labels=None, reload_weights_file=None, num_layers_unfreeze=0):#keep
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    num_layers_in_base_model = len(base_model.layers)
    base_model_layer_names = [k.name for k in base_model.layers]
    base_model.trainable = False  ##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$
    len_base_model = len(base_model.layers)
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]
    if reload_weights_file is not None:
        model.load_weights(reload_weights_file)  # load model weights

    # for i, k in enumerate(model.layers):
    #   if i >= len_base_model-5:
    #     k.trainable = True
    #   else:
    #     k.trainable = False
    relu_layers = []
    for i, k in enumerate(model.layers):
        if 'relu' in k.name.lower():
            relu_layers.append(i)
    relu_layers.append(9999999)
    # relu_layers = np.flip(np.asarray(relu_layers)+1)
    relu_layers = np.flip(np.asarray(relu_layers))

    # num_layers_unfreeze =1          #0 means freeze entire model 1 means base model forzen 2 one more (group)laye runfrozen etc
    for i, k in enumerate(model.layers):
        if i >= relu_layers[
            num_layers_unfreeze] and 'batchnorm' not in k.name.lower():  # LK:J:LKJD:LKJD:LKDJ:LDKJ:DLKJD:LKJD:LKJD:LKJD:LKDJ:LKDJLK:DJ
            k.trainable = True
        else:
            k.trainable = False

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    model.num_layers_in_base_model = num_layers_in_base_model
    model.base_model_layer_names = base_model_layer_names
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model

def foo_rename2(instr):#keep
    if isinstance(instr, str) and '/My Drive/' in instr:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]
    else:
        return instr


def make_D_local(D):#keep
    for key in D:
        if isinstance(D[key], str):
            D[key] = foo_rename2(D[key])
    for key in D['epoch_dict']:
        if isinstance(D['epoch_dict'][key], str):
            D['epoch_dict'][key] = foo_rename2(D['epoch_dict'][key])
    return D

def add_lstm_to_model(model_in, num_layers_in_base_model, base_learning_rate=10 ** -5, lstm_len=7):#keep
    # model_in.base_model_layer_names
    base_model = Model(model_in.input, model_in.layers[num_layers_in_base_model - 1].output)
    model_out = Sequential()
    model_out.add(TimeDistributed(base_model, input_shape=(lstm_len, 96, 96, 3)))
    model_out.add(TimeDistributed(Flatten()))
    model_out.add(LSTM(256, activation='relu', return_sequences=False))
    model_out.add(Dense(64, activation='relu'))
    # model_out.add(Dropout(.5))
    model_out.add(Dense(1, activation='sigmoid'))

    optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
    loss = keras.losses.BinaryCrossentropy()
    metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5), keras.metrics.AUC(name='auc')]
    model_out.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    return model_out


def rename_content_folder(instr):# keep
    if isinstance(instr, str) and '/content/' in instr:
        return '/Users/phil/Desktop/content' + instr.split('content')[-1]
    else:
        return instr


"""
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
########################################################################################################################
"""


from whacc.transfer_learning import save_and_plot_history
# train_and_val_dir
# base_test_dir
key_name = 'acc_test'
max_or_min = 'max'
models_2_load = [45]
# $%^&&&& user hard coded settings
patience = 8  # DETERMINES early stopping
save_freq = "epoch"  # leave this as epoch
period = 1  # DETERMINES how often it saves the checkpoints
epochs = 2  # DETERMINES how many epochs the model trains for if early stopping is never triggered
batch_size = 2 ** 7  # DETERMINES number of images per batch
save_best_only = True
monitor = 'val_loss'
###

lstm_len = 7
send_text('training started, you will receive a text when all training is done')

num_layers_to_unfreeze_list = [0]
for num_layers_unfreeze in num_layers_to_unfreeze_list:  # in the resnet base model
    # all_models_directory =  copy.deepcopy(train_and_val_dir)   # DETERMINES location for all model you will run #del
    # FOR EACH UNIQUE BASE DATSET start
    for TMP_basename in next(os.walk(train_and_val_dir))[1]:
        test_data_dir = base_test_dir + os.sep + TMP_basename  # # doesnt determine anything just filler, I define test data based on variable "test_path"
        unique_h5_train_val_dir = copy.deepcopy(
            TMP_basename)  # ^^^^^^^DETERMINES^^^^^^^ the name of the folder where each type of data is stored
        # FOR EACH UNIQUE BASE DATSET end
        for iiii, model_ind in enumerate(tqdm(models_2_load)):
            if 'ipynb_checkpoints' not in TMP_basename:
                data = all_data[model_ind]
                best_epochs = sorted_loadable_epochs(data, key_name, max_or_min)
                reload_model_epoch = best_epochs[0]
                D = make_D_local(copy.deepcopy(data['info']))
                test_path = utils.get_h5s(base_test_dir + os.sep + TMP_basename, 0)[0]

                LEARNING_RATE = D['base_learning_rate']
                # LEARNING_RATE = LEARNING_RATE/2
                LEARNING_RATE = 10 ** -5
                dropout_val = .1
                # righthere4
                model = re_build_model_TL(D['model_name_str'], D['class_numbers'],
                                          base_learning_rate=LEARNING_RATE,
                                          dropout_val=dropout_val, IMG_SIZE=D['IMG_SIZE'],
                                          reload_weights_file=D['epoch_dict'][reload_model_epoch],
                                          num_layers_unfreeze=num_layers_unfreeze)

                model = add_lstm_to_model(model, model.num_layers_in_base_model, base_learning_rate=LEARNING_RATE,
                                          lstm_len=lstm_len)

                # def add_LSTM_to_model(model, num_layers_in_base_model, base_model_layer_names):
                """
        image_tools.ImageBatchGenerator_simple(batch_size, [h5s], label_key = 'labels')
        3) ensure saving is correct saving the model to local and the moving it to its final destination, 
        4) ensure training val and test are using the correct simple generator (maybe dont need the test data??????? likely do though)
        5) copy all the alt labels to the H5s automatically  
        """

                label_ind = np.where(np.asarray(D['label_key_name']) == np.asarray(
                    list(model_maker.label_naming_shorthand_dict().keys())))[0][0]
                pred_key_save_name = 'MODEL_2_' + data[
                    'full_name'] + '__' + key_name + ' ' + max_or_min + '__epoch ' + str(
                    reload_model_epoch) + '__L_ind' + str(label_ind) + '__LABELS'
                print(pred_key_save_name)

                # FOR EACH UNIQUE MODEL start
                image_source_h5_directory_ending = D[
                    'image_source_h5_directory_ending']  # ^^^^^^^DETERMINES^^^^^^^ THE IMAGE SOURCE
                label_key = D[
                    'label_key_name']  # ^^^^^^^DETERMINES^^^^^^^ THE LABEL SOURCE choose the ind based on the print out
                # label_key = 'labels'#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                tz = pytz.timezone('America/Los_Angeles')
                loc_dt = pytz.utc.localize(datetime.utcnow())
                LA_TIME = loc_dt.astimezone(tz)
                todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")

                a = D['model_save_dir_checkpoints']
                b = '/'.join([k for k in a.split('/') if k != ''])
                b = b.split('all_models/')[-1].split('/')[1:]

                base_data = unique_h5_train_val_dir.split('data_')[-1]
                b.insert(4, todays_version)
                # b.insert(0, type_trained_on)
                b.insert(0, base_data)
                new_base_folder_name = "LSTM_mod_" + str(model_ind) + "_" + str(num_layers_unfreeze)
                b.insert(0, new_base_folder_name)  ##################
                end_save_dir = os.sep.join(b)
                model_save_dir_checkpoints = rename_content_folder('/content/' + end_save_dir)

                ################## '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'+end_save_dir

                Path(model_save_dir_checkpoints).mkdir(parents=True, exist_ok=True)

                # info_dict = get_automated_model_info_TL(train_and_val_dir, image_source_h5_directory_ending, test_data_dir, data_string_key = unique_h5_train_val_dir)
                info_dict = get_automated_model_info_TL(train_and_val_dir, image_source_h5_directory_ending, test_data_dir,
                                                     data_string_key='')

                # val_gen = image_tools.ImageBatchGenerator(batch_size, info_dict['h5_val'], label_key = label_key)
                val_gen = image_tools.ImageBatchGenerator_simple(batch_size, info_dict['h5_val'], label_key=label_key)
                # train_gen = image_tools.ImageBatchGenerator(batch_size, info_dict['h5_train'], label_key = label_key)
                train_gen = image_tools.ImageBatchGenerator_simple(batch_size, info_dict['h5_train'],
                                                                   label_key=label_key)
                # test_path = info_dict['h5_val']#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                # test_gen = image_tools.ImageBatchGenerator(batch_size, test_path, label_key = label_key)
                test_gen = image_tools.ImageBatchGenerator_simple(batch_size, test_path, label_key=label_key)
                # test_gen = val_gen#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                # test_gen = image_tools.ImageBatchGenerator(batch_size, '/content/small_test.h5', label_key = label_key)#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                # FOR EACH UNIQUE MODEL end

                train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']], label_key)
                val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']], label_key)
                labels = np.concatenate(
                    (train_labels, val_labels))  # flkjasklfjklasjdflkjaslkfjalksdjflasdfkasdfjalsdfkjasdflk
                labels = train_labels
                class_numbers = np.unique(labels)  #### can use label info here but no need
                rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
                class_weights = {i: rebalance[i] for i in class_numbers}

                callbacks = basic_callbacks(model_save_dir_checkpoints, monitor=monitor, patience=patience,
                                            save_best_only=save_best_only, save_weights_only=True,
                                            save_freq=save_freq, period=period)

                plot_callback = transfer_learning.save_and_plot_history(test_gen, key_name=label_key, plot_metric_inds=[
                    1])  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                plot_callback = save_and_plot_history(test_gen, key_name=label_key, plot_metric_inds=[
                    1])  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                callbacks.append(
                    plot_callback)  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                ###FITTTTTTT

                print(D['image_source_h5_directory_ending'], label_key)
                history = model.fit(train_gen,
                                    epochs=epochs,
                                    validation_data=val_gen,
                                    class_weight=class_weights,
                                    callbacks=callbacks)

                for k in info_dict.keys():
                    D[k] = info_dict[k]

                training_info = pack_training_info(plot_callback)
                foo_save_and_plot(training_info, plot_callback, os.path.dirname(model_save_dir_checkpoints) + os.sep)

                # for saving purposed
                D['base_learning_rate'] = LEARNING_RATE
                D['num_layers_unfreeze'] = num_layers_unfreeze
                D['dropout_val'] = dropout_val

                base_dir = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'

                model_save_dir_checkpoints_Gdrive = copy.deepcopy(model_save_dir_checkpoints)
                model_save_dir_checkpoints_Gdrive = base_dir + model_save_dir_checkpoints_Gdrive.split('/content')[-1]
                D['model_save_dir_checkpoints'] = model_save_dir_checkpoints_Gdrive
                D['model_save_dir'] = os.path.dirname(model_save_dir_checkpoints_Gdrive)

                # once finished transfer the completed trained model to the final directory in gdrive
                target_path = os.path.dirname(base_dir + os.sep + end_save_dir)
                Path(target_path).mkdir(parents=True, exist_ok=True)
                save_info_dict(D)  # save file after folder is created
                utils.copy_file_filter(os.path.dirname(model_save_dir_checkpoints), target_path)
                shutil.rmtree(os.path.dirname(model_save_dir_checkpoints))

send_text('finished with all of it')

def copy_file_filter(src, dst, keep_strings='', remove_string=None, overwrite=False,
                     just_print_what_will_be_copied=False, disable_tqdm=False, return_list_of_files=False):

utils.copy_file_filtercopy_file_filter(os.path.dirname(model_save_dir_checkpoints), target_path)




from whacc import utils
from whacc import image_tools
import tensorflow as tf


from tensorflow.keras.callbacks import ModelCheckpoint
# from keras.callbacks import ModelCheckpoint
from tensorflow import keras
# import keras


from sklearn.utils import class_weight
import numpy as np
import time
import os

split_h5_files = [
'/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH0667x170317_JON/training.h5',
'/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH0667x170317_JON/validation.h5'
]

all_models = utils.get_model_list('/Users/phil/Downloads/Colab data/model_iterations')
model_2_load = all_models[12]

# for timing \/
first_run_test_time = []
# for timing /\


# basic settings
epochs = 100
batch_size = 50
learning_rate = 10 ** -4
MOD_BASE_NAME = model_2_load.split(os.path.sep)[-1].split('.')[0]
# early stopping
monitor = 'val_loss'
patience = 10
start_tmp = time.time()

# ------
# CREATE TRAINING BATCH GENERATOR \/
my_training_batch_generator = image_tools.ImageBatchGenerator(batch_size, [split_h5_files[0]])
# my_validation_batch_generator
my_validation_batch_generator = image_tools.ImageBatchGenerator(batch_size, [split_h5_files[1]])
# CREATE TRAINING BATCH GENERATOR /\
# ------
# LOAD MODEL, MODEL SETTINGS \/
model = tf.keras.models.load_model(model_2_load)
# model = tf.keras.models.load_model('/content/temp_saves/savetest.hdf5')

model.optimizer.learning_rate = learning_rate  ###%%$^&$&&$
model.layers[0].trainable = False  # base
model.layers[1].trainable = False  #
model.layers[2].trainable = True  # class head

# callbacks = [keras.callbacks.EarlyStopping (monitor = 'val_loss', patience = 10 )] # Early stopping

# LOAD MODEL, MODEL SETTINGS /\
# ------
# RUN MODEL \/
start = time.time()
total_seconds = time.time() - start
print('total run time :' + str(round(total_seconds / 60)), ' minutes')
# Class imbalance weighting
all_y = utils.get_h5_key_and_concatenate(split_h5_files, key_name='labels')
rebalance = class_weight.compute_class_weight('balanced', [0, 1], all_y.flatten())
class_weights = {i: rebalance[i] for i in range(2)}

# Early stopping
callbacks = [keras.callbacks.EarlyStopping(monitor=monitor, patience=patience),
             ModelCheckpoint('/content/temp_saves/savetest.hdf5', save_best_only=True)]  ###%%$^&$&&$

history = model.fit(my_training_batch_generator, epochs=epochs,
                    validation_data=my_validation_batch_generator,
                    callbacks=callbacks,
                    class_weight=class_weights,
                    verbose=1)
# RUN MODEL /\
# ------
# save info for later reference \/
info_dict = {'monitor': callbacks[0].monitor,
             'patience': callbacks[0].patience,
             'epochs': epochs, 'total_seconds_to_retrain': total_seconds,
             'layers_re_trainable': [k.trainable for k in model.layers] * 1,
             're_learning_rate': model.optimizer.learning_rate}
# save info for later reference /\
#  predict for all regular H5s ** 8

first_run_test_time.append(time.time() - start_tmp)
# print(str((np.mean(first_run_test_time) * (L_TOTAL - len(first_run_test_time))) / 60) + ' minutes left ESTIMATED')



from whacc import utils
import h5py
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.patches as mpatches
import seaborn as sns


def foo_rename(instr):  # keep
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


wedgeprops = {'linewidth': 2.0, 'edgecolor': 'white',  'antialiased': True}
pctdistance=.5

colors = np.asarray(sns.color_palette("Paired")[:4])
colors = colors[[1, 0, 2, 3, ], :]
labels = ['regular\ntouch', 'augmented\ntouch', 'augmented\nno-touch', 'regular\nno-touch']





""" data from the boosted model"""
tvt = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/tvt_y.pkl'
tvt = utils.load_obj(tvt)
# arrange data
d = {'Training': [], 'Validation': [], 'Test': []}
d_len = {'Training': [], 'Validation': [], 'Test': []}
for i, k in enumerate(d):
    x = np.mean(tvt[i] == 1)
    d[k] = [x, 1 - x]
    d_len[k] = int(np.round(len(tvt[i]), -3) / 1000)
# test data in the above file was for my ref only and not for plotting
# below file is used for paper
full_test_labels = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/full_test_labels.pkl'
full_test_labels = utils.load_obj(full_test_labels)

    h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS.h5'
in_range = utils.getkey(h5_file, 'in_range_by_hand_192000')
no_hair_inds = utils.getkey(h5_file, 'no_hair_inds')
keep_inds = np.logical_and(no_hair_inds==1, in_range==1)==True

full_test_labels = full_test_labels[keep_inds.astype(bool)]

d_len['Test'] = int(np.round(len(full_test_labels), -3) / 1000)
x = np.mean(full_test_labels == 1)
d['Test'] = [x, 1 - x]




# make plot
pie, ax = plt.subplots(1, 3, figsize=[10, 6])
ax = ax.flatten()
for i, key in enumerate(d):
    ax[i].pie(x=d[key], autopct="%.1f%%", pctdistance=pctdistance, colors=colors[::3, :], wedgeprops=wedgeprops)
    ax[i].title.set_text(key + " Data\n" + str(d_len[key]) + "k frames")
# make legend and its data
patches = []
for c, l in zip(colors[::3], labels[::3]):
    patches.append(mpatches.Patch(color=c, label=l))
plt.legend(handles=patches, bbox_to_anchor=(0.9, 1), loc=2, borderaxespad=0, fontsize=10)






""" data from the CNN model"""
all_data = utils.load_obj(foo_rename('/content/gdrive/My Drive/colab_data2/all_data'))
main_mod = all_data[45]['info']
inds = [383810, 163120, 0]
d = {'Training': [], 'Validation': [], 'Test': []}
d_len = {'Training': [], 'Validation': [], 'Test': []}
keys = list(d.keys())
count = []
for i, k2 in enumerate(['train', 'val']):  # , 'test']):
    k = foo_rename(main_mod['h5_' + k2])
    print(k)
    with h5py.File(k, 'r') as h:
        L = len(h['labels'][:])
        count.append(L)
        d[keys[i]].append(np.sum(h['labels'][inds[i]:] != 0) / L)  # reg touch
        d[keys[i]].append(np.sum(h['labels'][:inds[i]] != 0) / L)  # aug touch
        d[keys[i]].append(np.sum(h['labels'][:inds[i]] == 0) / L)  # aug no-touch
        d[keys[i]].append(np.sum(h['labels'][inds[i]:] == 0) / L)  # reg no-touch
        d_len[keys[i]] = int(np.round(L, -3) / 1000)

fn2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/full_in_range_16_and_whacc_pred_no_hair.pkl'
yhat_d = utils.load_obj(fn2)
test_lab = yhat_d['labels'] * yhat_d['in_range']
d['Test'] = [np.mean(test_lab == 1), np.mean(test_lab != 1)]
d_len['Test'] = int(np.round(len(test_lab), -3) / 1000)

pie, ax = plt.subplots(1, 3, figsize=[10, 6])
ax = ax.flatten()

for i, key in enumerate(['Training', 'Validation']):
    ax[i].pie(x=d[key], autopct="%.1f%%", pctdistance=pctdistance, colors=colors, wedgeprops=wedgeprops)
    ax[i].title.set_text(key + " Data\n" + str(d_len[key]) + "k frames")
for i, key in enumerate(['Test']):
    ax[2].pie(x=d[key], autopct="%.1f%%", pctdistance=pctdistance, colors=colors[::3, :], wedgeprops=wedgeprops)
    ax[2].title.set_text(key + " Data\n" + str(d_len[key]) + "k frames")

# make legend and its data
patches = []
for c, l in zip(colors, labels):
    patches.append(mpatches.Patch(color=c, label=l))
plt.legend(handles=patches, bbox_to_anchor=(0.95, 1), loc=2, borderaxespad=0, fontsize=10)



from whacc import utils, image_tools
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.lines import Line2D

y = np.zeros(2001)
ymax = .1
y[290:310] = ymax
y[350:355] = ymax
y[570:600] = ymax
shift_zero_by = 190
fig = plt.figure(figsize=(14, 4))
colors = ['r', 'g', 'k'] # can change these as needed
for i, borders in enumerate([3, 80]):
    b = utils.inds_around_inds(y, borders * 2 + 1)
    tmp1, _ = utils.group_consecutives(b)
    for k in tmp1:
        k = np.asarray(k)-shift_zero_by
        plt.plot(k, [ymax +.03 + (i/40)]*len(k), colors[i])
plt.plot(y[shift_zero_by:], colors[i+1])
plt.ylim([-.1, 1])
plt.xlim([0, 550])

custom_lines = [Line2D([0], [0], color=colors[0], lw=2),
                Line2D([0], [0], color=colors[1], lw=2),
                Line2D([0], [0], color=colors[2], lw=2)]

plt.legend(custom_lines, ['3 border', '80 border', 'touch trace'])



#
#
# h5_in = '/Users/phil/Desktop/holy_test_set_10_percent_3lag.h5'
# y = image_tools.get_h5_key_and_concatenate(h5_in, '[0, 1]- (no touch, touch)')
#
# h5_in = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag/train_3lag.h5'
# y = image_tools.get_h5_key_and_concatenate(h5_in, 'labels')
#
# h5_in = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/3lag/small_test_3lag.h5'
# y = image_tools.get_h5_key_and_concatenate(h5_in, 'labels')
#
# utils.print_h5_keys(h5_in)
# """
# of course everything will be covered by 80 border because that is what the holy set is made up of!!
# replace the file with one of min or jons original data (ideally one with out any shift issues)
# """
#
# y = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
#        1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
#        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
#        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
#
#
# y = np.zeros(2001)
# ymax = .1
# y[290:310] = ymax
# y[350:355] = ymax
# y[570:600] = ymax
#
#
# plt.figure(figsize=(14, 4))
# colors = ['r', 'g', 'b', 'm']
# for i, borders in enumerate([3, 80]):
#
#     b = utils.inds_around_inds(y, borders * 2 + 1)
#     tmp1, _ = utils.group_consecutives(b)
#     for k in tmp1:
#         plt.plot(k, [ymax +.03 + (i/40)]*len(k), colors[i])
# plt.plot(y, 'k')
# plt.ylim([-.1, 1])
#
# from matplotlib.lines import Line2D
# custom_lines = [Line2D([0], [0], color='m', lw=2),
#                 Line2D([0], [0], color='b', lw=2),
#                 Line2D([0], [0], color='g', lw=2),
#                 Line2D([0], [0], color='r', lw=2),
#                 Line2D([0], [0], color='k', lw=2)]
#
#
# # fig, ax = plt.subplots()
# plt.legend(custom_lines, ['80 border', '40 border', '20 border', '3 border', 'touch trace'])
#
#
# x = np.zeros(201)
# x[90:110] = 1
# N = 161
# assert N / 2 != round(N / 2), 'N must be an odd number so that there are equal number of points on each side'
# cumsum = np.cumsum(np.insert(x, 0, 0))
# a = (cumsum[N:] - cumsum[:-N]) / float(N)
# a = np.where(a > 0)[0] + ((N - 1) / 2)
# a.astype('int')
#
#

##############





import matplotlib.pyplot as plt
from tqdm.auto import tqdm
from imgaug import augmenters as iaa  # optional program to further augment data

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted, ns
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py

from whacc import utils, analysis
import h5py
import matplotlib.pyplot as plt
import numpy as np
from sklearn import metrics
from pathlib import Path

import mat73
from whacc import utils
mat_file = '/Users/phil/Dropbox/U_191028_1154.mat'
data_dict = mat73.loadmat(mat_file)

# utils.info(data_dict)

# for k in ['R_ntk', 'S_ctk', 'c', 'cellNum', 'details', 'k', 'meta', 't', 'u', 'varNames', 'whisker']:
#     print('__________________________________________________________________')
#     print(k)
#     utils.info(data_dict['U'][0][k])
"""
LIST OC CELLS 
'R_ntk', 'S_ctk', 'c', 'cellNum', 'details', 'k', 'meta', 't', 'u', 'varNames', 'whisker'

--details-- 


"""

"""
contacts onset 
--  
 'firstTouchOnset',
 'firstTouchOffset',
 'firstTouchAll',
 'lateTouchOnset',
 'lateTouchOffset',
 'lateTouchAll',
 
whisker videos (ensure ell the data is 4000 long) so I can match with the whacc analysis

spikes 

"""
for cell_ind in range(len(data_dict['U'])):
    print('_______________')
    print(cell_ind)
    print(data_dict['U'][cell_ind]['details']['cellNum'])


cell_ind = 19
d = dict()

d['spikes'] = data_dict['U'][cell_ind]['R_ntk']
vars = ['thetaAtBase',
 'velocity', 'amplitude', 'setpoint', 'phase', 'deltaKappa', 'M0Adj', 'FaxialAdj', 'firstTouchOnset', 'firstTouchOffset',
 'firstTouchAll', 'lateTouchOnset', 'lateTouchOffset', 'lateTouchAll', 'PoleAvailable','beamBreakTimes']
for i, k in enumerate(vars):
    d[k] = data_dict['U'][cell_ind]['S_ctk'][i]

d['contacts'] = np.logical_or(d['firstTouchAll'], d['lateTouchAll'])

vars2 = ['trialNumsUseTrials', 'useTrials', 'whiskerTrialNumsUseTrials']
for i, k in enumerate(vars2):
    d[k] = data_dict['U'][cell_ind]['details'][k]

for i, k in enumerate(vars2):
    print('__________________')
    print(k)
    print(d[k].shape)
    print(d[k][-10:])
for k in ['cellCode', 'cellNum', 'depth', 'mouseName', 'sessionName', 'userRemovedTrials', 'whiskerTrialTimeOffset']:
    d[k] = data_dict['U'][cell_ind]['details'][k]
    print('__________________')
    print(k)
    print(d[k])

# tmp1 = utils.loadmat('/Users/phil/Desktop/ConTA_23.mat')
"""
utils.auto_combine_final_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/PHILLIP/processing/P1_FINISHED_MP4s/AH0688/170820')
"""

# utils.info(data_dict['U'][cell_ind]['meta'])
#
#
#
# x = np.pad(x, 4, mode='empty')
# x
#
#
#
# x = [0, 0, 1, 1, 1, 0, 0]
# start_pad = 4
# inds = [2, 4]
# # inds = 2
# end_pad = None
def cut_with_nans(x, inds, start_pad, end_pad = None):
    inds = np.asarray(utils.make_list(inds, True)) + start_pad
    if end_pad is None:
        end_pad = start_pad
    x = np.concatenate((np.ones(start_pad)*np.nan, x, np.ones(end_pad)*np.nan))
    x_out = []
    for k in inds:
        i1 = np.max([0, k-start_pad])
        i2 = k+end_pad+1
        x_out.append(x[i1:i2])
    return np.asarray(x_out)

# cut_with_nans(x, inds, start_pad, end_pad = None)


start_pad = 50
end_pad = 250
x_out = []
for x in tqdm(d['contacts'].T):
    inds = np.where(np.diff(np.concatenate(([0], x)))==1)[0]
    if len(inds) != 0:
        x_out.append(cut_with_nans(x, inds, start_pad, end_pad = end_pad))


x2 = np.vstack(x_out)
x3 = np.nanmean(x2, axis=0)

plt.plot(x3)


#
# plt.imshow(d['firstTouchAll'][400:700, :].T)
# plt.imshow(d['lateTouchAll'][400:700, :].T)



# plt.imshow(d['contacts'][400:700, :].T)


# np.where(d['firstTouchOnset'][:, 0]==1)[0]








from whacc import utils
mat_file = '/Users/phil/Dropbox/U_191028_1154.mat'
mat = utils.loadmat(mat_file)





bd = '/Users/phil/Dropbox/'
files = utils.get_files(bd, '*')

out = utils.lister_it(files, ['U_'])
out = utils.lister_it(out, ['.mat'])

for k in out:
    print(k)



from whacc import utils
import h5py
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.patches as mpatches


def foo_rename(instr):#keep
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]
wedgeprops={'linewidth': 3.0, 'edgecolor': 'white'}

all_data = utils.load_obj(foo_rename('/content/gdrive/My Drive/colab_data2/all_data'))
main_mod = all_data[45]['info']
#
# for k2 in ['train', 'val', 'test']:
#     k = foo_rename(main_mod['h5_'+k2])
#     exec(k2 + ' = "' + k + '"')
#     print(k)
#
# # training
# 383810 # aug - 3 border X 10
# 292217 # reg - 80 border (676027 - 383810)
# # validation
# 163120 # aug - 3 border X 10
# 118734 # regular - 80 border (281854 - 163120)
# # test
# 0      # aug
# 38884  #regular
#
#
# inds = [383810, 163120, 0]
# d = {'Training': [], 'Validation': [], 'Test': []}
# d_len = {'Training': [], 'Validation': [], 'Test': []}
# keys = list(d.keys())
# count = []
# for i, k2 in enumerate(['train', 'val']):#, 'test']):
#     k = foo_rename(main_mod['h5_'+k2])
#     print(k)
#     with h5py.File(k, 'r') as h:
#         L = len(h['labels'][:])
#         count.append(L)
#         d[keys[i]].append(np.sum(h['labels'][inds[i]:] != 0) / L) # reg touch
#         d[keys[i]].append(np.sum(h['labels'][:inds[i]] != 0) / L) # aug touch
#         d[keys[i]].append(np.sum(h['labels'][:inds[i]] == 0) / L) # aug no-touch
#         d[keys[i]].append(np.sum(h['labels'][inds[i]:] == 0) / L) # reg no-touch
#
# print(np.round(count, -3)) #array([676000, 282000,  39000])
#
import seaborn as sns
colors = np.asarray(sns.color_palette("Paired")[:4])
colors = colors[[1, 0, 2, 3,], :]
labels = ['regular\ntouch', 'augmented\ntouch', 'augmented\nno-touch', 'regular\nno-touch']
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d['Training'], autopct="%.1f%%", explode=[0.02]*4, labels=labels, pctdistance=0.5, colors= colors)
# plt.title("Training Data\n676k frames", fontsize=14)
#
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d['Validation'], autopct="%.1f%%", explode=[0.02]*4, labels=labels, pctdistance=0.5, colors= colors)
# plt.title("Validation Data\n282k frames", fontsize=14)
#
#
# # doing this the old way I would only get the 10% holy set because that is what the data is evaled on originally
# # this is all the test data without the hair data with samsons 1 session (no hair session) and andrews 2 sessions
# fn2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/full_in_range_16_and_whacc_pred_no_hair.pkl'
# yhat_d = utils.load_obj(fn2)
# yhat_d['frame_nums'], yhat_d['in_range']
# test_lab= yhat_d['labels'] * yhat_d['in_range']
# print(len(test_lab))
# d['Test'] = [np.mean(test_lab==1), np.mean(test_lab!=1)]
#
#
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d['Test'], autopct="%.1f%%", explode=[0.02]*2, labels=labels[::3], pctdistance=0.5, colors= colors[::3, :])
# plt.title("Testing Data\n780k frames", fontsize=14)
#
#
#








tvt = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/tvt_y.pkl'
tvt = utils.load_obj(tvt)
# arrange data
d = {'Training': [], 'Validation': [], 'Test': []}
d_len = {'Training': [], 'Validation': [], 'Test': []}
for i, k in enumerate(d):
    x = np.mean(tvt[i]==1)
    d[k] = [x, 1-x]
    d_len[k] = int(np.round(len(tvt[i]), -3)/1000)
# make plot
pie, ax = plt.subplots(1, 3, figsize=[10,6])
ax = ax.flatten()
for i, key in enumerate(d):
    ax[i].pie(x=d[key], autopct="%.1f%%", pctdistance=0.5, colors=colors[::3, :], wedgeprops=wedgeprops)
    ax[i].title.set_text(key+" Data\n"+str(d_len[key])+"k frames")
# make legend and its data
patches = []
for c, l in zip(colors[::3], labels[::3]):
    patches.append(mpatches.Patch(color=c, label=l))
plt.legend(handles=patches, bbox_to_anchor=(0.9, 1), loc=2, borderaxespad=0, fontsize=10)












inds = [383810, 163120, 0]
d = {'Training': [], 'Validation': [], 'Test': []}
d_len = {'Training': [], 'Validation': [], 'Test': []}
keys = list(d.keys())
count = []
for i, k2 in enumerate(['train', 'val']):#, 'test']):
    k = foo_rename(main_mod['h5_'+k2])
    print(k)
    with h5py.File(k, 'r') as h:
        L = len(h['labels'][:])
        count.append(L)
        d[keys[i]].append(np.sum(h['labels'][inds[i]:] != 0) / L) # reg touch
        d[keys[i]].append(np.sum(h['labels'][:inds[i]] != 0) / L) # aug touch
        d[keys[i]].append(np.sum(h['labels'][:inds[i]] == 0) / L) # aug no-touch
        d[keys[i]].append(np.sum(h['labels'][inds[i]:] == 0) / L) # reg no-touch
        d_len[keys[i]] = int(np.round(L, -3)/1000)

fn2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/full_in_range_16_and_whacc_pred_no_hair.pkl'
yhat_d = utils.load_obj(fn2)
test_lab= yhat_d['labels'] * yhat_d['in_range']
d['Test'] = [np.mean(test_lab==1), np.mean(test_lab!=1)]
d_len['Test']= int(np.round(len(test_lab), -3)/1000)


pie, ax = plt.subplots(1, 3, figsize=[10,6])
ax = ax.flatten()
for i, key in enumerate(['Training', 'Validation']):
    ax[i].pie(x=d[key], autopct="%.1f%%", pctdistance=0.5, colors=colors, wedgeprops=wedgeprops)
    ax[i].title.set_text(key+" Data\n"+str(d_len[key])+"k frames")
for i, key in enumerate(['Test']):
    ax[2].pie(x=d[key], autopct="%.1f%%", pctdistance=0.5, colors=colors[::3, :], wedgeprops=wedgeprops)
    ax[2].title.set_text(key+" Data\n"+str(d_len[key])+"k frames")

# make legend and its data
patches = []
for c, l in zip(colors, labels):
    patches.append(mpatches.Patch(color=c, label=l))
plt.legend(handles=patches, bbox_to_anchor=(0.95, 1), loc=2, borderaxespad=0, fontsize=10)







#
# key = 'Training'
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d[key], autopct="%.1f%%", explode=[0.02]*2, labels=labels[::3], pctdistance=0.5, colors= colors[::3, :])
# plt.title(key+" Data\n"+str(d_len[key])+"k frames", fontsize=14)
#
# key = 'Validation'
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d[key], autopct="%.1f%%", explode=[0.02]*2, labels=labels[::3], pctdistance=0.5, colors= colors[::3, :])
# plt.title(key+" Data\n"+str(d_len[key])+"k frames", fontsize=14)
#
# key = 'Test'
# pie, ax = plt.subplots(figsize=[10,6])
# plt.pie(x=d[key], autopct="%.1f%%", explode=[0.02]*2, labels=labels[::3], pctdistance=0.5, colors= colors[::3, :])
# plt.title(key+" Data\n"+str(d_len[key])+"k frames", fontsize=14)






# ax[0].legend(handles=patches, bbox_to_anchor=(1.02, 1), loc=2, borderaxespad=0, fontsize=10)
# # ax[i].legend(patches, loc="best")
# plt.legend(handles=patches, bbox_to_anchor=(1.02, 1), loc=2, borderaxespad=0, fontsize=50)

#
# patches, texts = plt.pie(sizes, colors=colors, startangle=90)
# plt.legend(colors[::3, :], labels, loc="best")
#
#     # ax1.title.set_text
#
#


#
# k = test
# utils.print_h5_keys(k)
#
#
# a1 = 0
# with h5py.File(k, 'r') as h:
#     print(h['labels'].shape)
#     imgs = h['images'][a1:a1+20]
# fig, ax = plt.subplots(5, 4)
# for i, a in enumerate(ax.flatten()):
#     a.imshow(imgs[i])


# # ind = int(281854//1.1)
# # a1 = 234878
# # a2 = 281854
# # a3 = int((a2-a1)/20)
# # with h5py.File(k, 'r') as h:
# #     print(h['labels'].shape)
# #     imgs = h['images'][a1:a2:a3]
# # fig, ax = plt.subplots(5, 4)
# # for i, a in enumerate(ax.flatten()):
# #     a.imshow(imgs[i])
# #
#
# #
# #
# # with h5py.File(k, 'r') as h:
# #     print(h['labels'].shape)
# #     imgs = h['images'][163120:163120+20]
# #     print(len(h['labels'][163120:]))
# # fig, ax = plt.subplots(5, 4)
# # for i, a in enumerate(ax.flatten()):
# #     a.imshow(imgs[i])
# #
# #
# #
# #
# #
# #
# #
# # utils.get_dict_info(main_mod)
# #
# #
# # utils.open_folder(foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data'))



from whacc import image_tools
from datetime import datetime

h5 = [
    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/3lag/small_test_3lag.h5']
h5 = [
    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/regular/train_regular.h5']

h5 = [
    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5']

lstm_len = 5
batch_size = 100
h5_file_list = h5
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
print(datetime.now() - start)

lstm_len = 5
batch_size = 400
h5_file_list = h5
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
print(datetime.now() - start)

lstm_len = 5
batch_size = 800
h5_file_list = h5
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
print(datetime.now() - start)

lstm_len = 5
batch_size = 1200
h5_file_list = h5
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
print(datetime.now() - start)

import tensorflow as tf

print("Num GPUs Available: ", len(tf.config.experimental.list_physical_devices('GPU')))

multiplier_1 = ((8.33 + 3.59) / 8.33)
multiplier_1 * 8 * 200 / 60

from whacc import utils, image_tools
from datetime import datetime
import numpy as np
import h5py

# righthere
h5_file_list = [
    "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/3lag/small_train_3lag.h5"]
h5_file_list = ['/Users/phil/Dropbox/Colab data/H5_data/regular/AH0698_170601_PM0121_AAAA_regular.h5']
lstm_len = 7
b = lstm_len // 2
batch_size = 4000
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
new_h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
h5creator = image_tools.h5_iterative_creator(new_h5)
for k in range(3):  # the issue here is that I created teh H5 file with the 3 lag with the noise image background to
    # that I didnt have to do it in the generator, but for this I dont do that so I create and image and it overlaps
    # making it so that the end of one trial merges with the next trial, this isnt too hard to fix
    # best to fix this after the file has been generated
    print(k)
    x, y = G.__getitem__(k)
    x = ((x + 1) / 2) * 255  # I trasnform it here no worries
    print(np.min(x), np.max(x))

    h5creator.add_to_h5(x, y)
h5creator.close_h5()
print(datetime.now() - start)
utils.copy_h5_key_to_another_h5(h5_file_list[0], new_h5, 'frame_nums')
# h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
# with h5py.File(h5, 'r+') as h:
#     h['frame_nums'] = h['frame_nums'][:3]
h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
"""%$%$%$%$%$%$%$%___ black out edges of the each trials start and end ___%$%$%$%$%$%$%$%"""

with h5py.File(h5, 'r+') as h:
    for i1, i2 in utils.loop_segments(h['frame_nums']):
        x = h['images'][i1:i2]
        print(x.shape)
        edge_ind = np.flip(np.arange(1, b + 1))
        for i in np.arange(1, b + 1):
            x[i - 1, :edge_ind[i - 1], ...] = np.zeros_like(x[i - 1, :edge_ind[i - 1], ...])
            x[-i, -edge_ind[i - 1]:, ...] = np.zeros_like(x[-i, -edge_ind[i - 1]:, ...])
        h['images'][i1:i2] = x
"""%$%$%$%$%$%$%$%___ black out edges of the each trails start and end ___%$%$%$%$%$%$%$%"""
""" I could also make this directly work in the generator by brabing the inds of these black ones and making them
black based on that when they come up"""
lstm_len = 5
b = lstm_len // 2
with h5py.File(h5,
               'r') as h:  # 0(0, 1) 1(0) 3998(-1) 3999(-2, -1) ...  4000(0, 1) 4001(0) 7998(-1) 7999(-2, -1) #0,0     0,1     1,0     3998,-1     3999,-2     3999,-1
    full_edges_mask = np.ones_like(h['images'][:b * 2])
    edge_ind = np.flip(np.arange(1, b + 1))
    for i in np.arange(1, b + 1):
        full_edges_mask[i - 1, :edge_ind[i - 1], ...] = np.zeros_like(full_edges_mask[i - 1, :edge_ind[i - 1], ...])
        full_edges_mask[-i, -edge_ind[i - 1]:, ...] = np.zeros_like(full_edges_mask[-i, -edge_ind[i - 1]:, ...])
    all_edges = []
    for i1, i2 in utils.loop_segments(h['frame_nums']):  # 0, 1, 3998, 3999 ; 4000, 4001, 7998, 7999; ...
        edges = (np.asarray([[i1], [i2 - b]]) + np.arange(0, b).T).flatten()
        all_edges.append(edges)
    all_edges = np.asarray(all_edges)

adjust_these_edge_frames = np.intersect1d(all_edges.flatten(), np.arange(i1, i2 + 1))
for atef in adjust_these_edge_frames:
    mask_ind = np.where(atef == all_edges)[1][0]
    out[mask_ind] = out[mask_ind] * full_edges_mask[mask_ind]

np.asarray([[i1], [i2]]) + np.arange(0, b)

print(start_mask.shape, end_mask.shape)
# h['images'][i1:i2] = x
"""%$%$%$%$%$%$%$%___ black out edges of the each trails start and end ___%$%$%$%$%$%$%$%"""

"""%$%$%$%$%$%$%$%"""
h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
import h5py
import matplotlib.pyplot as plt

utils.print_h5_keys(h5)
with h5py.File(h5, 'r') as h:
    print(h['images'].shape)
    print(h['frame_nums'][:])
    x = h['images'][0:8000]

# x = (x+1)/2
for ind1 in [3996, 3997, 3998, 3999, 4000, 4001, 4002]:
    y = x[ind1]
    plt.figure()
    for i, k in enumerate(y):
        print(all(k.flatten() == 0))
        plt.subplot(3, 3, i + 1)

        # plt.figure()
        plt.imshow(k)

h5 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/3lag/small_train_3lag.h5'
utils.print_h5_keys(h5)
with h5py.File(h5, 'r') as h:
    print(h['frame_nums'][:])

from whacc import image_tools

h5_file_list = [
    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/3lag/small_train_3lag.h5']
lstm_len = 5
batch_size = 100
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
    print(x.shape, y.shape)

"""asdkfja;lskdjfkl;asjdf;lkjas;ldkfjlaksdjfkl"""

from whacc import utils, image_tools
from datetime import datetime
import numpy as np
import h5py

h5_file_list = [
    "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/3lag/small_train_3lag.h5"]
# h5_file_list = ['/Users/phil/Dropbox/Colab data/H5_data/regular/AH0698_170601_PM0121_AAAA_regular.h5']
lstm_len = 5
batch_size = 125
G = image_tools.ImageBatchGenerator_LSTM(lstm_len, batch_size, h5_file_list, label_key='labels', IMG_SIZE=96)
start = datetime.now()
new_h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
h5creator = image_tools.h5_iterative_creator(new_h5)
for k in range(
        G.__len__()):  # the issue here is that I created teh H5 file with the 3 lag with the noise image background to
    # that I didnt have to do it in the generator, but for this I dont do that so I create and image and it overlaps
    # making it so that the end of one trial merges with the next trial, this isnt too hard to fix
    # best to fix this after the file has been generated
    print(k)
    x, y = G.__getitem__(k)
    x = ((x + 1) / 2) * 255
    print(np.min(x), np.max(x))

"""apsdjf;lajsdlfk;jals;kdfj;laksdjflkasjdflkajsdlkfjalksdjfl;kajsdfkl;jakls;dfjlkasdjfkl"""
from whacc import image_tools

h5 = '/Users/phil/Desktop/LSTM_small_train.h5'
G = image_tools.ImageBatchGenerator_simple(1000, h5)
for k in range(G.__len__()):
    x, y = G.__getitem__(k)
    print(x.shape, y.shape)

import os


def copy_folder_structure(src, dst):
    def ignore_files(dir, files):
        for f in files:
            print(f)
            if os.path.isfile(os.path.join(dir, f)):
                if f[0] != '.':
                    print(1)
                    new_dir = os.path.dirname(f)
                    Path(new_dir).mkdir(parents=True, exist_ok=True)
                    new_f = '____'.join(f.split('.'))+'.txt'
                    new_f = f
                    f = open(new_dir +new_f , "w+")
                return f
        return
        # return [f for f in files if os.path.isfile(os.path.join(dir, f))]

        shutil.copytree(src, dst, ignore=ignore_files)


src = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/FEATURE_DATA/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/data_ANM234232_140118_AH1026_AAAA/'
dst = '/Users/phil/Desktop/AAAAAAAAAAAAA'
copy_folder_structure(src, dst)


import os

def list_files(startpath):
    for root, dirs, files in os.walk(startpath):
        level = root.replace(startpath, '').count(os.sep)
        indent = ' ' * 4 * (level)
        print('{}{}/'.format(indent, os.path.basename(root)))
        subindent = ' ' * 4 * (level + 1)
        f = open(new_dir +new_f , "w+")
        for f in files:
            print('{}{}'.format(subindent, f))

def showFolderTree(path,show_files=False,indentation=2,file_output=False):
    """
    Shows the content of a folder in a tree structure.
    path -(string)- path of the root folder we want to show.
    show_files -(boolean)-  Whether or not we want to see files listed.
                            Defaults to False.
    indentation -(int)- Indentation we want to use, defaults to 2.
    file_output -(string)-  Path (including the name) of the file where we want
                            to save the tree.
    """


    tree = []

    if not show_files:
        for root, dirs, files in tqdm(os.walk(path)):
            level = root.replace(path, '').count(os.sep)
            indent = ' '*indentation*(level)
            tree.append('{}{}/'.format(indent,os.path.basename(root)))

    if show_files:
        for root, dirs, files in os.walk(path):
            level = root.replace(path, '').count(os.sep)
            indent = ' '*indentation*(level)
            tree.append('{}{}/'.format(indent,os.path.basename(root)))
            for f in files:
                subindent=' ' * indentation * (level+1)
                tree.append('{}{}'.format(subindent,f))

    if file_output:
        output_file = open(file_output,'w')
        for line in tqdm(tree):
            output_file.write(line)
            output_file.write('\n')
    else:
        # Default behaviour: print on screen.
        for line in tree:
            print(line)


import os
from whacc import utils
from tqdm.autonotebook import tqdm
# src_list = ['LIGHT_GBM', 'colab_data2', 'Colab data']
# for src in src_list:
#     src = '/Volumes/GoogleDrive-114825029448473821206/My Drive/'+ src
#     src = utils.norm_path(src)
#     save_loc = '/Users/phil/Desktop/gdrive_directory_saves/xxxxxxxxxxxxxxx.txt'
#     save_loc = save_loc.replace('xxxxxxxxxxxxxxx', os.path.basename(src))
#     showFolderTree(src,show_files=False,indentation=2,file_output=save_loc)
#



save_loc = '/Users/phil/Desktop/gdrive_directory_saves/xxxxxxxxxxxxxxx.txt'
utils.make_path(os.path.dirname(save_loc))
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/'
src_list = ['LIGHT_GBM', 'colab_data2', 'Colab data']

for src in src_list:
    src = bd+ os.sep+src+os.sep
    print(src)
    # src = utils.norm_path(src)
    save_loc = save_loc.replace('xxxxxxxxxxxxxxx', os.path.basename(src))
    showFolderTree(src,show_files=False,indentation=2,file_output=save_loc)




from pathlib import Path
import shutil
import os
from whacc import utils

from tqdm import tqdm


def copy_file_filter(src, dst, keep_strings='', remove_string=None, overwrite=False,
                     just_print_what_will_be_copied=False, disable_tqdm=False):
    src = src.rstrip(os.sep) + os.sep
    dst = dst.rstrip(os.sep) + os.sep

    all_files_and_dirs = utils.get_files(src, search_term='*')
    to_copy = utils.lister_it(all_files_and_dirs, keep_strings=keep_strings, remove_string=remove_string)

    if just_print_what_will_be_copied:
        _ = [print(str(i) + ' ' + k) for i, k in enumerate(to_copy)]
        return
    to_copy2 = []  # this is so I can tqdm the files and not the folders which would screw with the average copy time.
    for k in to_copy:
        k2 = dst.join(k.split(src))
        if os.path.isdir(k):
            Path(k2).mkdir(parents=True, exist_ok=True)
        else:
            to_copy2.append(k)

    for k in tqdm(to_copy2, disable=disable_tqdm):
        k2 = dst.join(k.split(src))
        if overwrite or not os.path.isfile(k2):
            Path(os.path.dirname(k2)).mkdir(parents=True, exist_ok=True)
            shutil.copyfile(k, k2)
        elif not overwrite:
            print('overwrite = False: file exists, skipping--> ' + k2)


copy_file_filter('/Users/phil/Desktop/FAKE_full_data', '/Users/phil/Desktop/aaaaaaaaaa', keep_strings='/3lag/',
                 remove_string=None, overwrite=True, just_print_what_will_be_copied=False)


from whacc import utils
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.image as mpimg
all_template_images = utils.get_files('/Users/phil/Desktop/trashTMP/', '*.png')

for i, k in enumerate(all_template_images):
    img = mpimg.imread(k)
    if i%3**2 == 0:
        fig, axs = plt.subplots(3, 3, figsize=[10, 10])
        axs = axs.flatten()
    axs[i%3**2].imshow(img)


np.arange(20)**2








import h5py
import copy
import numpy as np
from tkinter import *
from tkinter import ttk
from PIL import ImageTk, Image


def touch_gui(H5_file_name, label_read_key, image_read_key='images', label_write_key=None):
    """

    Parameters
    ----------
    H5_file_name :
        
    label_read_key :
        
    label_write_key :
         (Default value = None)

    Returns
    -------

    """
    print("""',' = -10   '.' = 10\n';' = -100  ''' = 100\n'[' = -1000 ']' = 1000""")
    if label_write_key is None:
        label_write_key = label_read_key
    global im_ind
    global h5
    global img
    global LABELS

    with h5py.File(H5_file_name, 'r+') as h5:
        def move(add_to):
            """

            Parameters
            ----------
            add_to :
                

            Returns
            -------

            """
            global im_ind
            global h5
            global img
            global LABELS
            ###
            im_ind = im_ind + add_to
            xx = 3  # how many pictures on each side of the center picture
            img = np.zeros(np.asarray(h5[image_read_key][0].shape) + np.asarray([10, 2, 0])).astype('int16')
            img = np.repeat(img, xx * 2 + 1, axis=1)
            img = np.concatenate((img, np.expand_dims(img[:, 0, :], axis=[1])), axis=1)
            img = img[:, 1:, :]
            W = h5[image_read_key][0].shape[1]
            for i, k in enumerate(index_out(h5[image_read_key], im_ind - xx, im_ind + xx + 1)):
                b = h5[image_read_key][k]
                l = LABELS[k]
                if l == -1:
                    pass
                elif l > 0.5:
                    img[:, i * (W + 2):(i + 1) * (W + 2), :] = np.expand_dims(np.asarray([0, 255, 0]), axis=[0, 1])
                elif l < 0.5:
                    img[:, i * (W + 2):(i + 1) * (W + 2), :] = np.expand_dims(np.asarray([255, 0, 0]), axis=[0, 1])
                img[1:1 + b.shape[0], 2 * i + 1 + b.shape[1] * i: 2 * i + 1 + b.shape[1] * (i + 1), :] = b
            a = np.ones([10, img.shape[1], 3]) * 255
            x2 = img.shape[1] / xx
            a[:, int(img.shape[1] / (xx * 2 + 1) * xx):int(img.shape[1] / (xx * 2 + 1) * (xx + 1)), :] = 0
            img = np.concatenate((img, a), axis=0)
            im = Image.fromarray(img.astype('uint8'))
            im = im.resize(np.flip(np.asarray(img.shape[:2]) * 3), Image.ANTIALIAS)
            image_tk = ImageTk.PhotoImage(image=im)

            label_1.configure(image=image_tk)
            label_1.image = image_tk

            ind2print = copy.deepcopy(im_ind) + 1
            if im_ind < 0:
                ind2print = len(LABELS) - (im_ind * -1) + 1
            text.configure(text=str(ind2print % len(LABELS)) + ' of ' + str(len(LABELS)))
            text.text = str(ind2print % len(LABELS)) + ' of ' + str(len(LABELS))

        def index_out(a, start, stop):
            """

            Parameters
            ----------
            a :
                
            start :
                
            stop :
                

            Returns
            -------

            """
            if stop <= start:
                stop += len(a)
            return np.arange(start, stop) % len(a)

        def move_right(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global im_ind
            move(1)

        def move_left(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global im_ind
            move(-1)

        def move_left10(event=None):
            global im_ind
            move(-10)

        def move_right10(event=None):
            global im_ind
            move(10)

        def move_left100(event=None):
            global im_ind
            move(-100)

        def move_right100(event=None):
            global im_ind
            move(100)

        def move_left1000(event=None):
            global im_ind
            move(-1000)

        def move_right1000(event=None):
            global im_ind
            move(1000)

        def switch_label(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global LABELS
            global im_ind
            if LABELS[im_ind] == -1:
                pass
            elif LABELS[im_ind] > .5:
                LABELS[im_ind] = 0
            elif LABELS[im_ind] < .5:
                LABELS[im_ind] = 1
            move(0)

        def make_1(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global LABELS
            global im_i
            LABELS[im_ind] = 1
            move(0)

        def make_0(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global LABELS
            global im_i
            LABELS[im_ind] = 0
            move(0)

        def make_neg1(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global LABELS
            global im_i
            LABELS[im_ind] = -1
            move(0)

        def save_foo(event=None):
            """

            Parameters
            ----------
            event :
                 (Default value = None)

            Returns
            -------

            """
            global LABELS
            neg_ones = np.where(LABELS == -1)
            LABELS = (LABELS > .5) * 1
            for neg_ind in neg_ones:
                LABELS[neg_ind] = -1
            try:
                del h5[label_write_key]
                # time.sleep(10)  # give time to process the deleted file... maybe???
                h5.create_dataset(label_write_key, data=np.float64(LABELS))
            except:
                h5.create_dataset(label_write_key, data=np.float64(LABELS))

        root = Tk()
        im = None
        LABELS = copy.deepcopy(h5[label_read_key][:])
        c = ttk.Frame(root, padding=(5, 20, 6, 0))
        c.grid(column=0, row=0, sticky=(N, W, E, S))
        root.grid_columnconfigure(0, weight=1)
        root.grid_rowconfigure(0, weight=1)

        im_ind = -1
        image_tk = ImageTk.PhotoImage(image=Image.fromarray(h5[image_read_key][im_ind]))

        set_1 = ttk.Button(c, text="set to 1", command=make_1)
        set_1.grid(column=2, row=0, sticky=S, pady=5, padx=2)
        root.bind('<Up>', make_1)

        set_0 = ttk.Button(c, text="set to 0", command=make_0)
        set_0.grid(column=2, row=0, sticky=S, pady=5, padx=2)
        root.bind('<Down>', make_0)

        set_neg1 = ttk.Button(c, text="set to -1", command=make_neg1)
        set_neg1.grid(column=2, row=0, sticky=S, pady=5, padx=2)
        root.bind('1', make_neg1)

        right_btn = ttk.Button(c, text="right", command=move_right)
        right_btn.grid(column=1, row=0, sticky=N, pady=0, padx=0)
        root.bind('<Right>', move_right)

        left_btn = ttk.Button(c, text="left", command=move_left)
        left_btn.grid(column=0, row=0, sticky=S, pady=0, padx=3)
        root.bind('<Left>', move_left)

        root.bind(',', move_left10)
        root.bind('.', move_right10)
        root.bind('<Shift_R>', move_right10)

        root.bind(';', move_left100)
        root.bind("""'""", move_right100)

        root.bind('[', move_left1000)
        root.bind(']', move_right1000)

        switch_lab_btn = ttk.Button(c, text="switch label", command=switch_label)
        switch_lab_btn.grid(column=2, row=0, sticky=S, pady=5, padx=5)
        root.bind('`', switch_label)
        #     root.bind('<1>',switch_label)#click mouse

        quit_btn = ttk.Button(c, text="SAVE", command=save_foo)
        quit_btn.grid(column=3, row=0, sticky=N, pady=5, padx=2)

        quit_btn = ttk.Button(c, text="QUIT", command=root.destroy)
        quit_btn.grid(column=4, row=0, sticky=N, pady=5, padx=2)

        label_1 = ttk.Label(c)
        label_1.grid(column=0, row=10, sticky=N, pady=5, padx=5, columnspan=10, rowspan=1)

        text = Label(c, text=str(im_ind % len(LABELS)) + ' of ' + str(len(LABELS)))
        text.place(x=100, y=0)
        root.mainloop()



import matplotlib.pyplot as plt
from whacc import utils
import numpy as np
from whacc import image_tools
from keras.preprocessing.image import ImageDataGenerator
import h5py
from imgaug import augmenters as iaa  # optional program to further augment data

import os
from PIL import Image, ImageEnhance

rc = {"axes.spines.left" : False,
      "axes.spines.right" : False,
      "axes.spines.bottom" : False,
      "axes.spines.top" : False,
      "xtick.bottom" : False,
      "xtick.labelbottom" : False,
      "ytick.labelleft" : False,
      "ytick.left" : False}
plt.rcParams.update(rc)

a = 662
b = 11
base_dir = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/'
all_imgs = []
in_list = utils.lister_it(utils.get_h5s(base_dir, 0), '/val_', 'ALT_LABELS')
in_list = [in_list[k] for k in [2, 0, 1]]
for h5_file in in_list:
    with h5py.File(h5_file, 'r') as hf:
        img = image_tools.img_unstacker(hf['images'][a:a+b], num_frames_wide=b)
        all_imgs.append(img)
img = np.vstack(all_imgs)

img1 = Image.fromarray(img)
converter = ImageEnhance.Color(img1)
fig, ax = plt.subplots(nrows=6, ncols=1,sharex=True, sharey=False)
for k in range(6):
    img2 = converter.enhance(k+1)
    ax[k].imshow(img2)

save_name = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/image_mod_types/image_example'
fig.set_size_inches(20*2, 15*2)
plt.savefig(save_name+'.pdf', dpi=600, transparent = True)
plt.close()




h5_file = in_list[1]
h5_file = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/val_3lag.h5'

datagen = ImageDataGenerator(rotation_range=360,  #
                            width_shift_range=.1,  #
                            height_shift_range=.1,  #
                            shear_range=.00,  #
                            zoom_range=.25,
                            brightness_range=[0.2, 1.2])  #
gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)

num_aug = 10
img_ind = 5
for seed in [2]: # 'best' random seeds to show all modifications [1, 2, 7, 13] 2 seems the best for example
    np.random.seed(seed)
    all_imgs = []
    with h5py.File(h5_file, 'r') as hf:
        image = hf['images'][a+img_ind-1]
        aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, -1)
        aug_img_stack = gaussian_noise.augment_images(aug_img_stack)
        all_imgs.append(aug_img_stack)
    example_augmented_image = image_tools.img_unstacker(np.squeeze(np.asarray(all_imgs)), num_aug/2)

    img1 = Image.fromarray(example_augmented_image)
    converter = ImageEnhance.Color(img1)
    fig, ax = plt.subplots(nrows=6, ncols=1,sharex=True, sharey=False)
    for k in range(6):
        img2 = converter.enhance(k+1)
        ax[k].imshow(img2)


save_name = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/image_mod_types/image_example_augmented_at_touch_frame_5_seed_'+str(seed)
fig.set_size_inches(20*2, 15*2)
plt.savefig(save_name+'.pdf', dpi=600, transparent = True)
plt.close()









h5_file = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/val_3lag.h5'
all_imgs = []
with h5py.File(h5_file, 'r') as hf:
        print(hf['labels'].shape)
        img = image_tools.img_unstacker(hf['images'][a:a+b], num_frames_wide=b)
        all_imgs.append(img)
img = np.vstack(all_imgs)

plt.imshow(img)





from whacc import image_tools, analysis, utils
import numpy as np
import matplotlib.pyplot as plt


f = '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/AH0698_170601_PM0121_AAAA_ALT_LABELS.h5'
a = utils.print_h5_keys(f, return_list=True, do_print=True)

a2 = a[88]
print(a2)
pred = image_tools.get_h5_key_and_concatenate([f], a2)
pred = ((pred > .5) * 1).flatten()
real = image_tools.get_h5_key_and_concatenate([f], a[105])get_class_info

frame_num_array = (np.ones(60) * 4000).astype(int)

a = analysis.error_analysis(real, pred, frame_num_array=frame_num_array)


print(set(a.all_error_type))
h5_img = '/Users/phil/Dropbox/Colab data/H5_data/regular/AH0698_170601_PM0121_AAAA_regular.h5'
pp = analysis.pole_plot(h5_img, pred_val=a.pred, true_val=a.real, figsize = [10, 5])


def display_error_types(pp, a, show_type, cnt = 0, edge_size = 2):
    certain_type = np.where(np.asarray(a.all_error_type) == show_type)[0]

    for k in range(len(a.all_error_type)):
        cnt+=1
        if cnt in certain_type:
            break
        else:
            print('ALL DONE')
            return None

    pp.current_frame = a.all_errors.copy()[cnt][0]-edge_size
    pp.len_plot = len(a.all_errors.copy()[cnt])+(edge_size*2)
    pp.plot_it()

    plt.plot([edge_size, edge_size], [-1, 10], 'g')
    plt.plot([pp.len_plot-edge_size-1, pp.len_plot-edge_size-1], [-1, 10], 'g')
    plt.title(a.all_error_type[cnt] + '   ' + str(a.all_error_nums[cnt]))
    print(cnt)
    return cnt

h5_img = '/Users/phil/Dropbox/Colab data/H5_data/regular/AH0698_170601_PM0121_AAAA_regular.h5'
pp = analysis.pole_plot(h5_img, pred_val=a.pred, true_val=a.real, figsize=[10, 5])
show_type = 'join'
cnt = -1

cnt = display_error_types(pp, a, show_type, cnt = cnt, edge_size = 2)



# H5_file_name = '/Users/phil/Dropbox/Autocurator/data/samsons_subsets/training_and_validation/AH1131X25032020ses335_subset.h5'
H5_file_name = '/Users/phil/Downloads/AH0667x170317_subset.h5'
label_read_key = 'labels'
label_write_key = 'labels'
from whacc.touch_curation_GUI import touch_gui
touch_gui(H5_file_name, label_read_key, image_read_key='images', label_write_key=label_write_key)






import copy

from whacc import utils, image_tools
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
import scipy.io as spio
import scipy.stats as stats

h5_in = '/Users/phil/Desktop/holy_test_set_10_percent_3lag.h5'
utils.print_h5_keys(h5_in)
y_onsets = image_tools.get_h5_key_and_concatenate(h5_in, '[0, 1]- (not onset, onset)')
y = image_tools.get_h5_key_and_concatenate(h5_in, '[0, 1]- (no touch, touch)')
frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')

c = ['r', 'k']
plt.figure()
for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
    plt.plot(np.arange(i1, i2), y[i1:i2], c[i % 2])

split_segs = np.asarray(utils.loop_segments(frame_nums, returnaslist=True))
y_cut = y[split_segs[0, 6]:split_segs[1, 6]][0:1150]  # cropped so that it is 10 touches all from the same video segment
y_onsets_cut = y_onsets[split_segs[0, 6]:split_segs[1, 6]][0:1150]
# plt.figure()
# plt.plot(y_cut, 'k')

d = {'real touch': y_cut, 'join': copy.deepcopy(y_cut), 'split': copy.deepcopy(y_cut), 'ghost': copy.deepcopy(y_cut),
     'miss': copy.deepcopy(y_cut), 'append_1': copy.deepcopy(y_cut), 'deduct_1': copy.deepcopy(y_cut),
     'append_2': copy.deepcopy(y_cut), 'deduct_2': copy.deepcopy(y_cut)}


d['split'][870:879] = 0
d['ghost'][920:926] = 1
d['miss'][797:824] = 0
d['join'][820:835] = 1

d['append_1'][796:801] = 1
d['append_2'][820:824] = 1
d['deduct_1'][798:802] = 0
d['deduct_2'][820:824] = 0


for i, k in enumerate(d):
    d[k] = d[k]+i*1.2

df = pd.DataFrame(d)
df.plot()
plt.xlim((750,950))
# plt.figure()
# df.plot('real touch')




key_in = 'append_1'
df.plot(y=['real touch', key_in], alpha = 0.5, color = ['k', 'g'])
b = np.where(df['real touch'] != df[key_in])[0]
b = np.unique([b+1] + [b-1])
b = b[b>=0]
# x = df[key_in].loc[b]

plt.plot(df[key_in].loc[b[0]:b[-1]], alpha = 0.5, color = 'r')
plt.xlim((750,950))


n = 3
x = np.reshape(np.repeat(y_cut, n), [-1, n]).T
sns.heatmap(x[:, 750:950], linewidths=2, linecolor='black')



        """[ 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]"""
a = np.tile([0,0,0,0,0,1,1,1,1,1,1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0], [7, 1]).astype('float32')

e0, e1 = .2, .8

a[1, 5:7] = e0# deduct
a[2, 3:5] = e1 # append

a[3, 8:10] = e0 # split
a[4, 1:4] = e1 # ghost

a[5, 15:19] = e0 # miss
a[6, 12:15] = e1 # join



a = 1*(a>.5).astype(float)
a[a==0] = .2

sns.heatmap(a, vmin=0, vmax=1,linewidths=1, cmap='Greys' , linecolor='white',square = True)
""""heat map of the types of touches example"""
# # save_name = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/touch types/heat map of the types of touches exampleV4.pdf'
# # plt.savefig(save_name, transparent=True)


# all below was the simulation code example
#
#
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#
# def get_points_edge(arr_in, edge = 31):
#     tmp1 = (edge//2) - np.arange(edge)
#     tmp2 = np.where(arr_in)[0]
#     out = (tmp2[:, None] - tmp1[None, :])#.astype('float32')
#     return out
#
# def extract_array(array_in, ind_in):
#     out = np.ones_like(ind_in)*np.nan
#     place_ind0 = np.nanargmin(np.abs(ind_in), axis=1)
#     place_ind1 = np.nanargmax(np.abs(ind_in), axis=1)
#     for i, (i1, i2, ind) in enumerate(zip(place_ind0, place_ind1, ind_in)):
#         out[i, i1:i2+1] = array_in[ind[i1]:ind[i2]+1]
#     return out
#
# def get_onset_or_offset(arr_in, neg1or1 = 1):
#     # neg1or1 = 1 # 1 is onset -1 is offset
#     arr_in = 1*(arr_in>.5)
#     touch_onset =  (1*(np.hstack(([0], np.diff(arr_in))) == neg1or1)).astype('float')
#     return touch_onset
#
# def get_stacked_spike_arrays(touch_onset, poisson_array, edge = 11):
#     ind_in = get_points_edge(touch_onset, edge = edge)
#     out = extract_array(poisson_array, ind_in)
#     return out
#
# def make_poisson(on_or_offset, low = 3, high = 10):
#     tmp = copy.deepcopy(on_or_offset)
#     print(low, high)
#     tmp[tmp == 0] = low
#     tmp[tmp == 1] = high
#     print(tmp)
#     poisson_array_out = np.random.poisson(tmp)
#     return poisson_array_out
#
#
# noise_add = 0
# neg1or1 = 1
# onset_real_tmp1 = get_onset_or_offset(a[0], neg1or1 = neg1or1)
# # onset_real_tmp += .1
# plt.plot(onset_real_tmp1, 'r--', alpha=1)
# filt = np.asarray([100, 40, 30, 20, 10, 4, 3, 1, 1])/40
# assert len(filt)%2 == 1, 'must be odd'
#
# onset_real_tmp = np.pad(onset_real_tmp1, len(filt)-1)
# onset_real_tmp[onset_real_tmp==0] = 0.1
# onset_real = np.convolve(onset_real_tmp, filt, mode='valid')
# for  k in range(10):
#     spike_array_source = np.random.poisson(onset_real)
#     if noise_add>0:
#         spike_array_source+=  np.random.poisson(noise_add, len(onset_real))
#     spike_array_source = spike_array_source[:len(onset_real_tmp1)]
#     # poisson_array = make_poisson(onset_real, low = 100,high = 100)
#     plt.plot(spike_array_source, 'b-', alpha=0.2)
# # can cut to length of the original onset_real_tmp1
# plt.plot(onset_real[:len(onset_real_tmp1)], 'k-', alpha=1)
#
#
# def create_error_array(touch_mat, operation, onset, center, num_to_operate = 1):
#     touch_mat = copy.deepcopy(touch_mat)
#     if operation == 1:#deduct 2
#         op_name = 'deduct'
#         touch_mat[:num_to_operate, onset:onset+2] = 0
#     elif operation==2:#append 2
#         op_name = 'append'
#         touch_mat[:num_to_operate, onset-2:onset] = 1
#     elif operation==3:#split 3 at center
#         op_name = 'split'
#         touch_mat[:num_to_operate, center-1:center+2] = 0
#
#     elif operation==4: #ghost edge at last 3 places
#         op_name = 'ghost'
#         touch_mat[:num_to_operate, -3:] = 1
#     elif operation==5:#miss
#         op_name = 'miss'
#         touch_mat[-num_to_operate:, :] = 0 # reversed this so that it is different than join cause
#     #     it was literally the exact same plot and you couldnt see join
#     elif operation==6:#join
#         op_name = 'join'
#         touch_mat[:num_to_operate, center:] = 1
#         touch_mat[1:num_to_operate+1, :center] = 1
#
#
#     else:
#         assert False, 'operation number not valid'
#     return touch_mat.flatten(), op_name
#
# def make_fake_touch_array(num_touches = 10, len_touch = 11, in_touch_array_len = 31):
#     # num_touches = 10; len_touch = 11; in_touch_array_len = 31
#     assert in_touch_array_len%2 ==1, 'must be odd'
#     assert len_touch%2 ==1, 'must be odd'
#
#     center = in_touch_array_len//2
#     edge_len = len_touch//2
#     onset =  center - edge_len
#
#     t = np.zeros([num_touches, in_touch_array_len])
#     t[:, onset:onset+len_touch] = 1
#     return t, onset, center
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#
#
# t, onset, center = make_fake_touch_array(num_touches = 4, len_touch = 11, in_touch_array_len = 31)
# plt.plot((1.1*-1)+t.flatten(), '-', label = 'real touch')
# for k in range(6):
#     t_mod, op_name = create_error_array(t, k+1, onset, center, num_to_operate = 1)
#     plt.plot((1.1*k)+t_mod, '-', label=op_name)
# plt.xlim([plt.xlim()[0], plt.xlim()[1]*1.5])
# plt.legend()
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#
# def poissonSpikeGen(fr, tSim, nTrials):
#   # the spike matrix spikeMat and a time vector tVec,
#   # a vector describing the time stamps for each column of the 2D matrix spikeMat.
#
#   dt = 1/1000; # s
#   tSim = tSim*dt
#   nBins = np.floor(tSim/dt);
#   spikeMat = np.random.rand(int(nTrials), int(nBins)) < fr*dt;
#   tVec = np.arange(0,tSim, dt)
#   return spikeMat, tVec
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# def get_SNR(a, win_size = 10, bl_move = 5):
#     center = a.shape[1]//2
#     signal = a[:, center:center+win_size]
#     baseline = a[:, center-win_size-bl_move:center-bl_move]
#     return signal.var()/baseline.var()
#
# def get_sd_for_signal(a, win_size = 10, bl_move = 5, return_inds = False):
#     center = a.shape[1]//2
#     signal = a[:, center:center+win_size]
#     baseline = a[:, center-win_size-bl_move:center-bl_move]
#     # print(signal.shape)
#     out = (signal - np.nanmean(baseline)) / np.std(baseline)
#     if return_inds:
#         return np.nanmean(out), [center, center+win_size, center-win_size-bl_move, center-bl_move]
#     else:
#         return np.nanmean(out)
#
# def get_sd_for_signal(a, win_size = 10, bl_move = 5, return_inds = False):
#     center = a.shape[1]//2
#     signal = a[:, center:center+win_size]
#     baseline = a[:, center-win_size-bl_move:center-bl_move]
#     # print(signal.shape)
#     out = (signal - np.nanmean(baseline)) / np.std(baseline)
#     if return_inds:
#         return np.nanmean(out), [center, center+win_size, center-win_size-bl_move, center-bl_move]
#     else:
#         return np.nanmean(out)
#
#
# def exponential_decay(a, b, N):
#     # a, b: exponential decay parameter
#     # N: number of samples
#     return a * (1-b) ** np.arange(N)
# high = 1
# len1 = 11
# tmp1 = exponential_decay(high, 0.2, len1)
# tmp2 = exponential_decay(high, 0.1, len1)
# plt.plot(tmp1)
# plt.plot(tmp2)
# plt.plot(tmp1+tmp2)
#
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# filt = tmp1+tmp2
# # make filter and poisson array
# # filt = np.asarray([100, 40, 30, 20, 10, 4, 3, 1, 1])/40
# # filt = np.asarray([100, 90, 70, 60, 20, 4, 3, 1, 1])/40
# # filt = np.repeat(filt, 3).flatten()
# # filt = exponential_decay(1, 0.1, 21)
# # filt = exponential_decay(2.5, 0.05, 41)
# """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# # 8*100 should be about the correct amount of touches for 100 trials about 8 per trial
#
#
# def y_square(l, r):
#     y = list(plt.ylim())
#     return (l, y[0]), r-l, y[1]-y[0]
#
# num_to_operate = 500
# num_touches = 1500
# baseline_p_num = .1
# signal_p_num = .5
# width_signal_to_extract = 51
# max_peak_probability = 0.99
# in_touch_array_len = 101
#
#
# t, onset, center = make_fake_touch_array(num_touches = num_touches, len_touch = 31, in_touch_array_len = in_touch_array_len)
# # noise_add = 1 # just for baseline adding there si still noise cause sampled form low alpha poisson
# onset_real_tmp1 = get_onset_or_offset(t.flatten(), neg1or1 = 1)# 1 for onset
# all_t = {'real touch':onset_real_tmp1.flatten()}
# for k in range(6): # save all onsets from altered touch arrays
#     t_mod, op_name = create_error_array(t, k+1, onset, center, num_to_operate = num_to_operate)
#     t_mod_onsets = get_onset_or_offset(t_mod.flatten(), neg1or1 = 1)# 1 for onset
#     all_t[op_name] = t_mod_onsets
#
# assert len(filt)%2 == 1, 'must be odd'
#
# onset_real_tmp = np.pad(onset_real_tmp1, len(filt)-1)
# onset_real_tmp[onset_real_tmp==0] = baseline_p_num
# onset_real_tmp[onset_real_tmp==1] = signal_p_num
#
# onset_real = np.convolve(onset_real_tmp, filt, mode='valid')
# # tmp1 = onset_real - np.min(onset_real)
# # onset_real = max_peak_probability*(onset_real/np.max(onset_real))
# # onset_real = onset_real_tmp[:len(onset_real_tmp1)]
#
# # plt.plot(onset_real_tmp1[:100]+0.1)
# # plt.plot(onset_real[:100])
#
#
#
# # tmp1 = []
# # for k in onset_real[:in_touch_array_len]:
# #     [tmp2, _] = poissonSpikeGen(k, 1, num_touches)
# #     tmp1.append(tmp2.flatten())
# #
# # spike_array_source = np.hstack(tmp1)
#
#
# spike_array_source = onset_real
#
# # spike_array_source = np.random.poisson(onset_real)
#
# # if noise_add>0:
# #     spike_array_source+=  np.random.poisson(noise_add, len(onset_real))
#
# spike_array_source = 1*spike_array_source[:len(onset_real_tmp1)]
# onset_real_final = onset_real[:len(onset_real_tmp1)]
# # poisson_array = onset_real # set this to see just the pure lamda values of each type
# final_spikes = []
# fig = plt.figure()
# for i, k in enumerate(all_t):
#
#     out = get_stacked_spike_arrays(all_t[k], spike_array_source, edge = width_signal_to_extract)
#     # plt.plot(np.std(out, axis  =0))
#     # print(out.shape, k)
#     y = np.nanmean(out, axis = 0)
#     x_vals = np.arange(len(y))-len(y)//2
#     # sem =stats.sem(out, axis = 0)
#     # if i==0:
#     #     subtract_me = 0
#     # y=y-subtract_me#+(i/5)
#     # print(np.mean(sem))
#     # print(np.std(out, axis  =0))
#
#
#     sd, inds_2_plot = get_sd_for_signal(out, win_size = 10, bl_move = 5, return_inds = True)
#     sd = np.std(out, axis  =0)
#     snr = get_SNR(out)
#
#     tmp1 = plt.plot(x_vals, y, label = k)
#     # plt.fill_between(x_vals, y-sd, y+sd, color=tmp1[0]._color, alpha=0.2)
#
#
#     final_spikes.append(out)
# plt.legend()
#
#
#
# inds_2_plot = np.asarray(inds_2_plot)-len(y)//2
# plt.gca().add_patch(plt.Rectangle(*y_square(*inds_2_plot[:2]),fill=True, color='g', alpha=0.5, figure=fig, zorder=-1000, edgecolor=None))
# plt.gca().add_patch(plt.Rectangle(*y_square(*inds_2_plot[2:]),fill=True, color='r', alpha=0.5, figure=fig, zorder=-1000, edgecolor=None))
#
#
#
# from matplotlib.patches import Rectangle
#
# # def foo_square(fig, l, r, alpha, color):
# #     y = list(plt.ylim())
# #     print((l, y[0]), r-l, y[1]-y[0])
# #     plt.gca().add_patch(plt.Rectangle((l, y[0]), r-l, y[1]-y[0], fill=True,
# #                                       color='g', alpha=0.5, zorder=1000,
# #                                   transform=fig.transFigure, figure=fig))
#
#
# def get_sd_for_signal(a, win_size = 10, bl_move = 5):
#     center = a.shape[1]//2
#     signal = a[:, center:center+win_size]
#     baseline = a[:, center-win_size-bl_move:center-bl_move]
#     # print(signal.shape)
#     print(np.nanmean(signal), np.nanmean(baseline), np.std(baseline))
#     out = (signal - np.nanmean(baseline)) / np.std(baseline)
#     return np.nanmean(out)
#
#
# tmp1 = np.zeros(1000)
# tmp1[:500] = 1
# np.std(tmp1)
#
#
#
#
# zscore = get_sd_for_signal(out, win_size = 10, bl_move = 5)
#
#
# a = out;  win_size = 10; bl_move = 5
#
# center = a.shape[1]//2
# signal = a[:, center:center+win_size]
# baseline = a[:, center-win_size-bl_move:center-bl_move]
#
#
# signal.var()/baseline.var()
#
# mbl = np.nanmean(baseline, axis = 1)
# msig = np.nanmean(signal, axis = 1)
# (signal - mbl[:, None])/mbl[:, None]
#
# plt.plot(np.nanmean(a, axis = 0))
#
# np.nanmean(signal - mbl[:, None])
#
# # here peak response is   max (mean ( signal-mean(Baseline in that trial) / mean (baseline in that trial) ) ) in a certain time frame
#
#
# foo_square(fig, inds_2_plot[0], inds_2_plot[1], .5, 'r')
# plt.show()
# inds_2_plot
#
#
#
# fig, ax = plt.subplots(nrows=1, ncols=1)
# ax.plot(np.cumsum(np.random.randn(100)))
# fig.patches.extend([plt.Rectangle((0.25,0.5),0.25,0.25,
#                                   fill=True, color='g', alpha=0.5, zorder=1000,
#                                   transform=fig.transFigure, figure=fig)])
#
#
#
# ylim = list(plt.ylim()) ; ylim[0]  = 0 ; plt.ylim(ylim)
#
#
#
#
#
# for k in final_spikes:
#     print(get_sd_for_signal(k))
#
#
#
# plt.plot(final_spikes[0][:100, :])
#
#
# plt.plot(spike_array_source)
# plt.plot(onset_real_final)
#
# # plt.plot(onset_real_tmp)
# # plt.plot(onset_real)
#
# """
# make smoothing windows of some ms after the touch onset
# """
# all_out = []
# num_count = 100
# for _ in range(num_count):
#     for k in a:
#         onset_tmp = get_onset_or_offset(k, neg1or1 = neg1or1)
#         out = get_stacked_spike_arrays(onset_tmp, spike_array_source)
#         all_out.append(np.nanmean(out, axis = 0))
# tmp1 = np.asarray(all_out)
# # tmp1 = np.tile((np.tile(np.arange(7), [11, 1]).T), [2, 1])
# tmp2 = np.reshape(tmp1, [num_count, 7, -1])
# tmp3 = np.nanmean(tmp2, axis = 0)
# plt.plot(tmp3.T)
#
# plt.legend(['normal', 'deduct', 'append', 'split', 'ghost', 'miss','join' ])
# # _____
#
# plt.plot(tmp1)
# plt.plot(onset_real)
# # _____
# """
# not getting the results I expected with these fake neurons because they are too close together, also missing a touch doesn;t
# affect the peak because all peaks are the same so an average doesnt affect it
# IRL peaks are variable tho, also SE you'll see a difference there it will be less tight
# """
#
#
# touch_onset =  (1*(np.hstack(([0], np.diff(arr_in))) == 1)).astype('float')
# touch_onset =  (1*(np.hstack((np.diff(arr_in), [0])) == neg1or1)).astype('float')
#
# get_points_edge(touch_onset, edge = 11)
# # find min and max and locations, pop them in a nan init array
#
#
# #
#
# # def get_points_edge(arr_in, edge = 31, neg1or1 = 1):
# #     touch_onset = 1*(np.hstack(([0], np.diff(arr_in))) == neg1or1)
# #     touch_onset = np.pad(touch_onset, edge)
# #     print(touch_onset.shape)
# #     x = inds_around_inds(touch_onset, edge)
# #     print(x)
# #     return x#np.reshape(x, [-1, edge])
#
#
# x = get_points_edge(a[0, :], 101)
#
# arr_in = a[0, :]; neg1or1 = 1
# touch_onset = np.hstack(([0], np.diff(arr_in))) == neg1or1
#
#
#
#
# np.pad([1, 2, 1], 1)





import matplotlib.pyplot as plt
from tqdm.auto import tqdm

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted, ns
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py
from imgaug import augmenters as iaa  # optional program to further augment data

import lightgbm as lgb

import shap

from whacc.figures import pretty_plots
pretty_plots


def load_selected_features(h5_in, feature_list = None, feature_index=None, time_index=None, return_labels = False, label_key = 'labels'):
    """

    Parameters
    ----------
    h5_in : full directory (or list of full directories of multiple h5s) of the full feature h5 file 84,009
    feature_index : bool array of len 84,009

    Returns
    -------

    """

    def indexit(arr, index):
        out = []
        for k in index:
            out.append(arr[k])
        return np.asarray(out)

    def extract_indexed_features(x, all_x_in, feature_index_in):
        if len(x.shape) > 1:
            inds = feature_index_in[:x.shape[1]]
            del feature_index_in[:x.shape[1]]
            x = x[:, inds]
            if all_x_in is None: # init
                all_x_in = x
            else:
                all_x_in = np.hstack((all_x_in, x))
        else:
            inds = feature_index_in.pop(0)  # single true or false to include the "TOTAL" variables
            if inds:
                if all_x_in is None: # init
                    all_x_in = x[:, None]
                else:
                    all_x_in = np.hstack((all_x_in, x[:, None]))
        return all_x_in, feature_index_in

    d = utils.load_feature_data()
    if feature_list is None:
        feature_list = d['feature_list_unaltered']
    if feature_index is None:
        feature_index= d['final_selected_features_bool']
        # feature_index = utils.get_selected_features(greater_than_or_equal_to=4)
    feature_index = utils.make_list(feature_index)
    feature_index = copy.deepcopy(feature_index)

    if isinstance(h5_in, list):
        all_x = []
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_selected_features(k, feature_list, feature_index, time_index, return_labels, label_key)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        if return_labels:
            all_y = np.hstack(all_y)
            return all_x, all_y
        return all_x

    all_x = None
    for k in tqdm(feature_list):
        if time_index is not None:
            x = []
            with h5py.File(h5_in, 'r') as h:
                for i in time_index:
                    x.append(h[k][i])
            x = np.asarray(x)
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')

        all_x, feature_index = extract_indexed_features(x, all_x, feature_index)

    if return_labels:
        all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key).astype('float32')
        all_y = np.hstack(all_y)
        if time_index is not None:
            all_y = indexit(all_y, time_index)
        return all_x, all_y
    return all_x




"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$ _______________________ $$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""


def load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0, label_key='labels'):
    if isinstance(h5_in, list):
        all_x = [];
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=split_int_n_chunks,
                                     split_chunk_ind=split_chunk_ind, label_key=label_key)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        all_y = np.hstack(all_y)
        return all_x, all_y
    all_x = None
    for k in feature_list:
        if all_x is None:
            all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype(
                'float32')
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype('float32')
            if len(x.shape) > 1:
                all_x = np.hstack((all_x, x))
            else:
                try:
                    all_x = np.hstack((all_x, x[:, None]))
                except:
                    all_x = np.hstack((all_x[:, None], x[:, None]))

    all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[split_chunk_ind::split_int_n_chunks].astype(
        'float32')
    return all_x, all_y


def rm_nan_rows(a, a2=None):
    indx = ~np.isnan(a).any(axis=1)
    if a2 is None:
        return a[indx, :]
    else:
        return a[indx, :], a2[indx]


def get_feature_data_names(feature_list, n=2048):
    featuredata_names = [k.replace('FD__', '').replace('____', '') for k in feature_list]
    final_feature_names = []
    names = []
    nums = []
    for i, (k1, k2) in enumerate(zip(featuredata_names, feature_list)):
        if '_TOTAL_' in k2:
            names.append(k1)
            nums.append(np.arange(1))
            final_feature_names.append(k1)
        else:

            names.append(np.repeat(k1, n))
            nums.append(np.arange(n))
            final_feature_names.append([i1 + '_' + str(i2) for i1, i2 in zip(names[-1], nums[-1])])
    out = [np.hstack(k) for k in [final_feature_names, names, nums, featuredata_names]]
    out[0] = list(out[0])
    out[-1] = list(out[-1])
    return out


# h5_in = '/Users/phil/Desktop/LIGHT_GBM/reg_80_border_and_andrew_both_samson_both__test_num_0_percent_15.h5'
h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/FEATURE_DATA/reg_80_border_and_andrew_both_samson_both__test_num_0_percent_15.h5'
# mod_fn = '/Users/phil/Desktop/LIGHT_GBM/model_saves/model_num_0_of_10.pkl'
mod_fn = '/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/final_model/light_GBM_model/FINAL_WHACC_MODEL_016_model.pkl'

with open(mod_fn, 'rb') as f:
    model = pickle.load(f)

explainer = shap.TreeExplainer(model)

# from whacc.feature_maker import get_feature_data_names
feature_list = natsorted(utils.lister_it(utils.print_h5_keys(h5_in, 1, 0), keep_strings='FD__'))
final_feature_names, feature_names, feature_nums, feature_list_short = get_feature_data_names(feature_list)
x_train, y_train = load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0)


cut_index = 1030 + np.asarray([-50, 50])

plt.plot(y_train[cut_index[0]:cut_index[1]])

starti = 88
sns.heatmap(x_train[:-41][cut_index[0]:cut_index[1], starti::2048//4].T)


'''
before these are all randomized i need to reference the final version that is very big 
i want fast touches with unique sidedness on each side 
access to images 
images that are good quality like jons or mine

'''
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$ SELECTING AN INFORMATIVE TOUCH TRACE $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

# h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'
h5_in = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'
# here I am look for a good slice of touches in the orignal H5 file
# training
383810 # aug - 3 border X 10
292217 # reg - 80 border (676027 - 383810)
y = image_tools.get_h5_key_and_concatenate(h5_in)[383810:]
smooth_by = 151
ind = 0

y2 = np.convolve(np.diff(y), np.ones(smooth_by)/smooth_by)
bst_inds = np.flip(np.argsort(y2))
add_to = smooth_by//2
fig, ax = plt.subplots(5, 4)
for ind, ax2 in enumerate(ax.flatten()):
    ind+=100
    ax2.plot(y[bst_inds[ind]-add_to:bst_inds[ind]+add_to])

ind = 115
add_to = 200
plt.plot(y[bst_inds[ind]-add_to:bst_inds[ind]+add_to])

plt.plot(y[271792:271792+200])

plt.plot(y[271792:271792+200-50])
# 271792+383810

with h5py.File(h5_in, 'r') as h:
    test_image = h['images'][bst_inds[ind]+383810]
plt.imshow(test_image)
# CHECKING THE MAKE SURE THE IMAGES LOOK GOOD
from whacc import analysis
pp = analysis.pole_plot(h5_in)
pp.current_frame = bst_inds[ind]+383810-20
pp.plot_it()
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$ MAKE AN EXAMPLE H5 FOR SHOWING FEATURE ENGINEERING $$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'
# h5_new = '/Users/phil/Desktop/feature_example2.h5'
h5_new = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/DATA/figure_data/features/feature_example.h5'
utils.print_h5_keys(h5_in)
with h5py.File(h5_in, 'r') as h:
    with h5py.File(h5_new, 'w') as h2:
        h2['images'] = h['images'][655102:656252]
        h2['labels'] = h['labels'][655102:656252]
        h2['frame_nums'] = [len(h2['labels'][:])]

with h5py.File(h5_new, 'r') as h:
    y3 = h['labels'][:]
    print(h['frame_nums'][:])


from whacc import model_maker
# convert to feature data
# h5_3lag = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag.h5'
h5_feature_data = h5_new.replace('.h5', '_feature_data.h5')
in_gen = image_tools.ImageBatchGenerator(500, h5_new, label_key='labels')
RESNET_MODEL = model_maker.load_final_model()
utils.convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)


# h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
# generate all the modified features (41*2048)+41 = 84,009

from whacc.feature_maker import convert_h5_to_feature_h5, standard_feature_generation2, standard_feature_generation
standard_feature_generation(h5_feature_data)


# from natsort import natsorted, ns
# xxxx = natsorted(xxxx, alg=ns.REAL)
d = utils.load_obj('/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/feature_data/feature_data_dict.pkl')

fi_path ='/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/feature_index_for_feature_selection/only_gain_more_than_2_features_bool_2105_features.npy'
feature_index = np.load(fi_path)

# all_x, all_y = load_selected_features(h5_feature_data, feature_list = d['feature_list_unaltered'], feature_index=feature_index, return_labels = True, label_key = 'labels')

with h5py.File(h5_feature_data ,'r') as h:
    all_x = []
    for k in d['feature_list_unaltered']:
        x = h[k][:]
        if len(x.shape)==1:
            x = x[:, None]
        all_x.append(x)
all_x = np.hstack(all_x)

all_x = all_x[:, feature_index]
# all_x = utils.load_selected_features(h5_feature_data)

all_x.shape

with h5py.File(h5_feature_data, 'r+') as h:
    h['final_features_2105'] = all_x


utils.info(utils.h5_to_dict(h5_feature_data, None))

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$ NOW PLOT IT TO SHOW THE READER  what the features look like$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""


"""$$$$$$$$$$$$$$$$$$$$$$$$$ grab the correct index $$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
import scipy.cluster.hierarchy as sch
def cluster_corr(corr_array, max_div_by = 2):
    pairwise_distances = sch.distance.pdist(corr_array)
    linkage = sch.linkage(pairwise_distances, method='complete')
    cluster_distance_threshold = pairwise_distances.max()/max_div_by
    idx_to_cluster_array = sch.fcluster(linkage, cluster_distance_threshold, criterion='distance')
    idx = np.argsort(idx_to_cluster_array)
    if isinstance(corr_array, pd.DataFrame):
        return corr_array.iloc[idx, :].T.iloc[idx, :]
    return corr_array[idx, :][:, idx], idx

h5_feature_data = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/DATA/figure_data/features/feature_example_feature_data.h5' #### NOTE replace this text only in this file "/Users/phil/Desktop/feature_example2_feature_data".h5'
utils.print_h5_keys(h5_feature_data)

with h5py.File(h5_feature_data, 'r') as h:
    d = dict()
    for k in h.keys():
        d[k] = copy.deepcopy(h[k][500:500+150])

# with h5py.File(h5_feature_data, 'r') as h:
#     d = dict()
#     for k in h.keys():
#         d[k] = copy.deepcopy(h[k][:])

def foo_neuron_heatmap(FD, inds, labels, only_return_data = False, linewidth=3):
    labels = np.asarray(labels).astype(float)
    # sns.heatmap(cc)
    data = FD[:, inds].T
    data = data-np.min(data)
    data = data/np.max(data)
    if only_return_data:
        return data
    fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
    plt.sca(ax[0])
    ax_heat = sns.heatmap(data, cbar=False)
    x = np.linspace(0, 2047, 5).astype(int)
    ax_heat.set_yticks(x)
    ax_heat.set_yticklabels(x+1)
    plt.ylabel('2048 CNN output neurons')
    plt.sca(ax[1])
    labels[labels==0] = np.nan
    plt.plot(labels, 'k-', label='touch trace', linewidth=linewidth)
    plt.ylim([0, 1.2])
    ax[1].set_yticklabels([])
    x = np.arange(0, 151, 50).astype(int)
    ax[1].set_xticks(x)
    ax[1].set_xticklabels(x)
    plt.xlabel('time (ms)')
    plt.tight_layout()
    plt.legend(bbox_to_anchor=(.8, 1.65), loc=2, borderaxespad=0, fontsize=10)
    plt.sca(ax[0])
    return data

# """$$$$$$$$$$$$$$$$$$$$$$$$$ to plot each $$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#
# labels = d['labels']
# FD = d['FD__original']
# FD[0, :] = FD[0, :]+0.00000001 # so that CC works, prevents values form one neuron all being the same
# # 1 method use clustering
# cc = np.corrcoef(FD, rowvar=False)
# cc, inds = cluster_corr(cc, 2)
#
#
# data = foo_neuron_heatmap(FD, inds, labels)
#
# from matplotlib.patches import Rectangle
# plt.gca().add_patch(Rectangle((40 - .5, 755 - .5), 30, 50, facecolor='none'))
#
#
# ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')
#
# plt.ylim([780+25, 780-25])
# plt.xlim([40, 70])
# plt.figure()
#
# sns.heatmap(data[780-25:780+25, 40:70])
"""$$$$$$$$$$$$$$$$$$$$$$$$$ to plot each $$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

labels = d['labels']
FD = d['FD__original']
FD[0, :] = FD[0, :] + 0.00000001  # so that CC works, prevents values form one neuron all being the same
# 1 method use clustering
cc = np.corrcoef(FD, rowvar=False)
cc, inds = cluster_corr(cc, 2)

x_left_corner = 40 - 10 - 5
y_left_corner = 755 - 15

x_len = 30 + 10 + 30 + 40
y_len = 50 + 30

data = foo_neuron_heatmap(FD, inds, labels, linewidth=5)

from matplotlib.patches import Rectangle

plt.gca().add_patch(Rectangle((x_left_corner - .0, y_left_corner - .0), x_len, y_len, facecolor='none'))

fig_bd = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/feature_engineering/'
plt.savefig(os.sep.join([fig_bd, 'OG 2048']) + '.pdf', transparent=True, format='pdf')

# ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')

plt.ylim([y_left_corner, y_left_corner + y_len])
plt.xlim([x_left_corner, x_left_corner + x_len])
fig_bd = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/feature_engineering/'
plt.savefig(os.sep.join([fig_bd, 'OG 2048 zoom']) + '.pdf', transparent=False, format='pdf')


plt.figure(figsize=[3.2, 3])
sns.heatmap(data[y_left_corner:y_left_corner + y_len, x_left_corner: x_left_corner + x_len])
plt.savefig(os.sep.join([fig_bd, 'clipped 2048 example2']) + '.pdf', transparent=True, format='pdf')

# plt.show(block=False)
plt.close('all')

# plt.legend(bbox_to_anchor=(.8, 1.85), loc=2, borderaxespad=0, fontsize=10)


# fd = d['FD__FD__original_diff_periods_-50____']
# foo_neuron_heatmap(fd, inds, labels)
#
# fd = d['FD__FD__original_diff_periods_-2____']
# foo_neuron_heatmap(fd, inds, labels)


keys = utils.print_h5_keys(h5_feature_data, 1, 0)
keys = natsorted(utils.lister_it(keys, 'FD__', 'TOTAL'), alg=ns.REAL)

cut_list = []
spacer_width = 10
spacer = np.ones([y_len, spacer_width])
for k in keys:
    fd = d[k]
    data = foo_neuron_heatmap(fd, inds, labels, only_return_data = True)
    cut_list.append(data[y_left_corner:y_left_corner + y_len, x_left_corner: x_left_corner + x_len])
    cut_list.append(spacer)

n = 1
figsize = [16/n, 5]
plt.figure(figsize=figsize)
x = np.hstack(cut_list[:16*2-1])
ax = sns.heatmap(x)
plt.savefig(os.sep.join([fig_bd, 'diff heatmap example']) + '.pdf', transparent=True, format='pdf')
plt.close()

figsize = [7/n, 5]
plt.figure(figsize=figsize)
x = np.hstack(cut_list[16*2-1:23*2-1])
ax = sns.heatmap(x)
plt.savefig(os.sep.join([fig_bd, 'smooth heatmap example']) + '.pdf', transparent=True, format='pdf')
plt.close()

figsize = [7/n, 5]
plt.figure(figsize=figsize)
x = np.hstack(cut_list[23*2-1:30*2-1])
ax = sns.heatmap(x)
plt.savefig(os.sep.join([fig_bd, 'standard deviation heatmap example']) + '.pdf', transparent=True, format='pdf')
plt.close()

figsize = [10/n, 5]
plt.figure(figsize=figsize)
x = np.hstack(cut_list[30*2-1:40*2-1])
ax = sns.heatmap(x)
plt.savefig(os.sep.join([fig_bd, 'shift heatmap example']) + '.pdf', transparent=True, format='pdf')
plt.close()

keys[:16] # diff
keys[16:23] # rolling mean
keys[23:30] # rolling std
keys[30:40] # shift

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$ PLOT TOTAL STD FEATURES   $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
keys = utils.print_h5_keys(h5_feature_data, 1, 0)

keys = natsorted(utils.lister_it(keys, 'TOTAL'), alg=ns.REAL)
total_type_features = []
for k in keys:
    fd = d[k]
    total_type_features.append(fd)

total_type_features = np.vstack(total_type_features)

key_labels = []
for k in keys:
    k2 = ' '.join(k.split('FD__original_')[-1].split('_')[:4])
    key_labels.append(k2)
    print(k2)
key_labels[-1] = 'Original'

fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
plt.sca(ax[0])

ax_heat = sns.heatmap(total_type_features, cbar=False, yticklabels=key_labels)
ax_heat.set_yticklabels(key_labels, fontsize= 7)

# x = np.linspace(0, 40, 5).astype(int)
# ax_heat.set_yticks(x)
# ax_heat.set_yticklabels(x)
plt.ylabel('TOTAL SD features (2048 reduced to 1)')
plt.sca(ax[1])
plt.plot(labels, 'k-', label='touch trace')
ax[1].set_yticklabels([])
x = np.arange(0, 151, 50).astype(int)
ax[1].set_xticks(x)
ax[1].set_xticklabels(x)
plt.xlabel('time (ms)')
plt.tight_layout()
plt.legend(bbox_to_anchor=(.8, 1.65), loc=2, borderaxespad=0, fontsize=10)
plt.sca(ax[0])


plt.savefig(os.sep.join([fig_bd, 'Total SD features V2']) + '.pdf', transparent=True, format='pdf')

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$ PLOT 2105 -- final features features   $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# h5_feature_data = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/DATA/figure_data/features/feature_example_feature_data.h5' #### NOTE replace this text only in this file "/Users/phil/Desktop/feature_example2_feature_data".h5'
# # utils.standard_feature_generation2()
# d = utils.h5_to_dict(h5_feature_data, None)
# utils.info(d)
def foo_norm(x):
    x = x-np.min(x)
    x = x/np.max(x)
    return x
# FD = d['final_3095_features']
FD = d['final_features_2105']
FD[0, :] = FD[0, :]+0.0000001 # to avoid true divide error
# FD = FD[:, ::10]
labels = d['labels']
FD2 = copy.deepcopy(FD)
for i, k in enumerate(FD.T):
    FD2[:, i] = foo_norm(k)
FD = FD2
# FD[0, :] = FD[0, :]+0.0000001 # to avoid true divide error
cc = np.corrcoef(FD, rowvar=False)
cc, inds = cluster_corr(cc, 1.5)
data = FD[:, inds].T
sns.heatmap(cc)

fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
plt.sca(ax[0])

ax_heat = sns.heatmap(data, cbar=False)
# ax_heat.set_yticklabels(key_labels, fontsize= 7)

# x = np.linspace(0, 40, 5).astype(int)
# ax_heat.set_yticks(x)
# ax_heat.set_yticklabels(x)
plt.ylabel('2105 selected features')
plt.sca(ax[1])
plt.plot(labels, 'k-', label='touch trace')
ax[1].set_yticklabels([])
x = np.arange(0, 151, 50).astype(int)
ax[1].set_xticks(x)
ax[1].set_xticklabels(x)
plt.xlabel('time (ms)')
plt.tight_layout()
plt.legend(bbox_to_anchor=(.8, 1.65), loc=2, borderaxespad=0, fontsize=10)
plt.sca(ax[0])

plt.savefig(os.sep.join([fig_bd, 'final 2105 features, clusters and normed']) + '.pdf', transparent=True, format='pdf')








# sns.heatmap(total_type_features)
#
# sns.heatmap(FD)
#
# lu = []
# for k in FD.T:
#     lu.append(len(np.unique(k)))
# plt.plot(np.sort(lu))
#
# np.sum(np.asarray(lu)==1)
#
# from whacc import analysis
# file_name = 'ANM234232_140120_AH1030_AAAA_a_3lag.h5'
# file_name = 'AH0407_160613_JC1003_AAAC_3lag.h5'
# file_name = utils.lister_it(utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/'), file_name)
#
#
# a = analysis.pole_plot(file_name[0])
# ind = 14467 # 14467 to 14567
# a.current_frame = ind
# a.plot_it()
#
# cut_index = [ind-150, ind+150]
# plt.plot(np.arange(*cut_index), a.true_val[cut_index[0]:cut_index[1]])
#
#
# cut_index = [ind+70, ind+140]  # [14537, 14607]
# print(np.diff(cut_index))
# plt.plot(np.arange(*cut_index), a.true_val[cut_index[0]:cut_index[1]])
#
#
#
#
#
#
#
# def load_data(h5_in, feature_list, start_ind, end_ind, label_key='labels'):
#     if isinstance(h5_in, list):
#         all_x = [];
#         all_y = []
#         for k in h5_in:
#             tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=start_ind,
#                                      split_chunk_ind=end_ind, label_key=label_key)
#             all_x.append(tmp_x)
#             all_y.append(tmp_y)
#             del tmp_x, tmp_y
#
#         all_x = np.vstack(all_x)
#         all_y = np.hstack(all_y)
#         return all_x, all_y
#     all_x = None
#     for k in feature_list:
#         if all_x is None:
#             all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[start_ind:end_ind].astype('float32')
#         else:
#             x = image_tools.get_h5_key_and_concatenate(h5_in, k)[start_ind:end_ind].astype('float32')
#             if len(x.shape) > 1:
#                 all_x = np.hstack((all_x, x))
#             else:
#                 all_x = np.hstack((all_x, x[:, None]))
#
#     all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[start_ind:end_ind].astype('float32')
#     return all_x, all_y
#
#
#
# ## a) use this range [14537, 14607]
# base_name = 'AH0407_160613_JC1003_AAAC_3lag.h5'
# # /Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Jon/AH0407/160613/AH0407x160609-144.mp4
# img_h5 = utils.lister_it(utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/'), base_name)
# feature_h5 = utils.lister_it(utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/FEATURE_DATA/colab data/curation_for_auto_curator/DATA_FULL_in_range_only/data_AH0407_160613_JC1003_AAAC/3lag/'), base_name)
# inds = [14419, 16417]
# with h5py.File(img_h5[0], 'r') as h:
#     images = h['images'][inds[0]:inds[1]]
#     labels = h['labels'][inds[0]:inds[1]]
#     frame_nums = np.asarray([len(labels)])
# with h5py.File(feature_h5[0], 'r') as h:
#     FD__original = h['FD__original'][inds[0]:inds[1]]
#
# with h5py.File('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature.h5', 'w') as h:
#     h['images'] = images
#     h['labels'] = labels
#     h['frame_nums'] = frame_nums
#     h['FD__original'] = FD__original
#     h['base_name'] = base_name.encode("ascii", "ignore")
#     h['img_h5'] = img_h5[0].encode("ascii", "ignore")
#     h['feature_h5'] = feature_h5[0].encode("ascii", "ignore")
#
#
# #
# # for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
# #     if i1<=14587 and i2>14587:
# #         print(i)
# #         print(i1, i2)
# #         break
# #
# # frame_nums[12]
# #
# #
# #
# # feature_list = natsorted(utils.lister_it(utils.print_h5_keys(h5_in, 1, 0), keep_strings='FD__'))
# # final_feature_names, feature_names, feature_nums, feature_list_short = get_feature_data_names(feature_list)
# # all_x, all_y = load_data(feature_h5[0], feature_list, inds[0], inds[1], label_key='labels')
# #
# # all_y[inds[0]:inds[1]]
# # utils.print_h5_keys(feature_h5[0])
# ## b) get full 2048 features from resnet heatmap it
#
# ## c) take one example neuron and plot all normal feature transformations, group the transforms base on type (shift,
# # SD, diff etc.)
#
# ## d) use b) to show how I do the TOTAL transforms
